/* file connection.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <gtk/gtk.h>
#include <assert.h>
#include "global.h"
#include "icon.h"
#include "command.h"

static GtkWidget *field_user;
static GtkWidget *field_password;
static GtkWidget *field_server;
static GtkWidget *field_ip_port;
/****************************************************************************/
static void nc81_field_user_changed_callback(GtkWidget *field, gpointer data)
{
	gchar* value;
	value = gtk_entry_get_text(GTK_ENTRY(field_user));
	nc81_set_user(value);
}
/****************************************************************************/
static void nc81_field_password_changed_callback(GtkWidget *field, gpointer data)
{
	gchar* value;
	value = gtk_entry_get_text(GTK_ENTRY(field_password));
	nc81_set_password(value);
}
/****************************************************************************/
static void nc81_field_server_changed_callback(GtkWidget *field, gpointer data)
{
	gchar* value;
	value = gtk_entry_get_text(GTK_ENTRY(field_server));
	nc81_set_server(value);
}
/****************************************************************************/
static void nc81_field_ip_port_changed_callback(GtkWidget *field, gpointer data)
{
	gchar* value;
	value = gtk_entry_get_text(GTK_ENTRY(field_ip_port));
	nc81_set_ip_port(value);
}
/*****************************************************************************/
static GtkWidget *make_connection_box()
{
	GtkWidget* container;
	GtkWidget* table;
	GtkWidget* widget;
	GtkObject* adj;

	container = gtk_frame_new("Connection");
	table = gtk_table_new(4, 2, FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(table), 10);
	gtk_container_add(GTK_CONTAINER(container), table);

	/* user name */
	widget = gtk_label_new("User name");
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 0, 1, 0, 1);
	widget = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 0, 1);
	gtk_widget_set_usize(widget, 80, 0);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_config_user);
	field_user=widget;
	gtk_signal_connect(GTK_OBJECT(field_user), "changed",
			GTK_SIGNAL_FUNC(nc81_field_user_changed_callback), NULL);

	/* password */
	widget = gtk_label_new("Password");
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 0, 1, 1, 2);
	widget = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 1, 2);
	gtk_widget_set_usize(widget, 80, 0);
	gtk_entry_set_visibility(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_config_password);
	field_password=widget;
	gtk_signal_connect(GTK_OBJECT(field_password), "changed",
			GTK_SIGNAL_FUNC(nc81_field_password_changed_callback), NULL);

	/* server name */
	widget = gtk_label_new("Server");
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 0, 1, 2, 3);
	widget = gtk_entry_new();
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 2, 3);
	gtk_widget_set_usize(widget, 80, 0);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_config_server);
	field_server=widget;
	gtk_signal_connect(GTK_OBJECT(field_server), "changed",
			GTK_SIGNAL_FUNC(nc81_field_server_changed_callback), NULL);

	/* IP port */
	widget = gtk_label_new("IP port");
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 0, 1, 3, 4);

	adj = gtk_adjustment_new(1, 0, 65536, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_table_attach_defaults(GTK_TABLE(table), widget, 1, 2, 3, 4);
	gtk_widget_set_usize(widget, 80, 0);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_config_ip_port);
	field_ip_port=widget;
	gtk_signal_connect(GTK_OBJECT(field_ip_port), "changed",
			GTK_SIGNAL_FUNC(nc81_field_ip_port_changed_callback), NULL);

	return container;
}
/*****************************************************************************/
static gint button_close_clicked(GtkWidget* button, gpointer data)
{
	nc81_main_load(nc81_form_main_menu);
	return 0;
}

/*****************************************************************************/
GtkWidget* nc81_connection_new(void)
{
	GtkWidget *vbox;
	GtkWidget *widget;

	vbox = gtk_vbox_new(FALSE,1);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

	/* connexion parameters */
	widget = make_connection_box();
	gtk_box_pack_start(GTK_BOX(vbox), widget, TRUE, FALSE,1);

	/* close button */
	widget = gtk_button_new_with_label("Close");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE,1);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_close_clicked), NULL);

	gtk_widget_show_all(vbox);
	return vbox;
}
/*****************************************************************************/
