/* file exp_job_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget *field_name;
static GtkWidget *field_description;
static GtkWidget *field_path;
static GtkWidget *field_argument;
static GtkWidget *field_wad; /* wait all dependencies */
static GtkWidget *field_oec; /* on error continue */
static GtkWidget *field_start_tempo;
static GtkWidget *field_end_tempo;
static GtkWidget *field_state;
static GtkWidget *field_enabled;
static GtkWidget *field_exit_code;


/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */
	if (strcmp(fields[0], "Job Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);

	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_description), fields[1]);

	if (strcmp(fields[0], "Enabled") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_enabled), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "State") == 0)
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(field_state)->entry), 
				fields[1]);

	if (strcmp(fields[0], "Exit code") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_exit_code), fields[1]);

	if (strcmp(fields[0], "Wait All Dependencies") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_wad), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "On Error Continue") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_oec), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "Start Tempo") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_start_tempo), 
				fields[1]);

	if (strcmp(fields[0], "End Tempo") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_end_tempo), fields[1]);

	if (strcmp(fields[0], "Path") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_path), fields[1]);

	if (strcmp(fields[0], "Argument") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_argument), fields[1]);
}
/****************************************************************************/
/* this function updates the job fields */
static void nc81_ref_task_edit_update(void)
{
	nc81_command_send("xji \"%d\" \"%t\" \"%j\"", 
			field_fill_callback, NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gboolean changed;
	gboolean bool_value;
	char* char_value;
	char* exit_code;

	/* path field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_path)));
	if (changed == TRUE)
	{
		char_value = gtk_entry_get_text(GTK_ENTRY(field_path));
		nc81_command_send("xjsp \"%d\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			char_value,NULL,NULL);
	}

	/* argument field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_argument)));
	if (changed == TRUE)
	{
		char_value = gtk_entry_get_text(GTK_ENTRY(field_argument));
		nc81_command_send("xjsa \"%d\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			char_value,NULL,NULL);
	}

	/* enabled field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_enabled)));
	if (changed == TRUE)
	{
		bool_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_enabled));
		nc81_command_send("xjse \"%d\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(bool_value == TRUE)?"true":"false",
			NULL,NULL);
	}

	/* state field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(GTK_COMBO(field_state)->entry)));
	if (changed == TRUE)
	{
		char_value = gtk_entry_get_text(GTK_ENTRY(
					GTK_COMBO(field_state)->entry));
		if (strcmp(char_value, "aborted") == 0)
		{
			exit_code = gtk_entry_get_text(
					GTK_ENTRY(field_exit_code));
			nc81_command_send(
				"xju \"%d\" \"%t\" \"%j\" \"%x\" \"%y\"", 
				field_name_valid_callback, NULL, 
				char_value,
				exit_code,
				NULL);

		} else {
			nc81_command_send("xju \"%d\" \"%t\" \"%j\" \"%x\"", 
				field_name_valid_callback, NULL, 
				char_value,
				NULL,NULL);
		}
	}


	/* TODO do not close if error */
	nc81_main_load(nc81_form_exploitation_job_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_job_view);
	return 0;
}
/****************************************************************************/
static void nc81_field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}

/****************************************************************************/
/* this function creates a widget which edit an exploitation job */
GtkWidget* nc81_exp_job_edit_new(void)
{
	GtkWidget *container;
	GtkWidget *note_book;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GList *items;

	container = gtk_vbox_new(FALSE, 2);
	note_book = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(container), note_book, TRUE, TRUE, 1);

	/* page General */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("General"));

	/* job name field */
	field_name = gtk_entry_new();
	gtk_widget_set_sensitive(field_name, FALSE);
	nc81_field_new_pack("Job", field_name, GTK_BOX(vbox));

	/* description field */
	field_description = gtk_entry_new();
	gtk_widget_set_sensitive(field_description, FALSE);
	nc81_field_new_pack("Job", field_description, GTK_BOX(vbox));

	/* path */
	field_path = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_path), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Path", field_path, GTK_BOX(vbox));

	/* argument */
	field_argument = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_argument), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Argument", field_argument, GTK_BOX(vbox));

	/* page Activity */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 3);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("Activity"));

	/* enabled field */
	widget = gtk_check_button_new_with_label("Enabled");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_enabled= widget;

	/* state field */
	field_state= gtk_combo_new();
	gtk_object_set_user_data(GTK_OBJECT(field_state), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("State", field_state, GTK_BOX(vbox));
	items = NULL;
	items = g_list_append(items, "canceled");
	items = g_list_append(items, "ignore");
	items = g_list_append(items, "aborted");
	items = g_list_append(items, "ready_to_run");
	items = g_list_append(items, "terminated");
	gtk_combo_set_popdown_strings(GTK_COMBO(field_state), items);

	/* exit code */
	field_exit_code = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_exit_code), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Exit code", field_exit_code, GTK_BOX(vbox));

	/* wad field */
	widget = gtk_check_button_new_with_label("Wait all dependencies");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	gtk_widget_set_sensitive(widget, FALSE);
	field_wad = widget;

	/* oec field */
	widget = gtk_check_button_new_with_label("On error continue");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	gtk_widget_set_sensitive(widget, FALSE);
	field_oec = widget;

	/* page tempo */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("Tempo"));
	/* start tempo */
	field_start_tempo = gtk_entry_new();
	gtk_widget_set_sensitive(field_start_tempo, FALSE);
	nc81_field_new_pack("Start tempo", field_start_tempo, GTK_BOX(vbox));

	/* end tempo */
	field_end_tempo = gtk_entry_new();
	gtk_widget_set_sensitive(field_end_tempo, FALSE);
	nc81_field_new_pack("End tempo", field_end_tempo, GTK_BOX(vbox));

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, FALSE, 1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	nc81_ref_task_edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_path), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_argument), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_enabled), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(field_state)->entry), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	return container;
}
/****************************************************************************/
