/* file global.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include "global.h"
#include "command.h"

gboolean small = FALSE;
gchar* nc81_ref_name;
gchar* nc81_task_name;
gchar* nc81_job_name;
gchar* nc81_day_name;
gchar* nc81_target_name;
gchar* nc81_queue_name;
gchar* nc81_rule_name;
gchar* nc81_user_name;
gchar* nc81_variable_name;

gchar* nc81_config_user;
gchar* nc81_config_password;
gchar* nc81_config_server;
gchar* nc81_config_ip_port;
/****************************************************************************/
GtkWidget *nc81_button_new_with_icon(gchar* title, nc81_icon_type icon_type)
{
	GtkWidget* button;
	GtkWidget* hbox;
	GtkWidget* label;
	GtkWidget* pixmap;

	button = gtk_button_new();
	hbox = gtk_hbox_new(FALSE,1);
	gtk_container_add(GTK_CONTAINER(button), hbox);

	assert( nc81_icon_pixmap_array[icon_type] != NULL);
	assert( nc81_icon_bitmap_array[icon_type] != NULL);
	pixmap = gtk_pixmap_new(
			nc81_icon_pixmap_array[icon_type], 
			nc81_icon_bitmap_array[icon_type]);
	gtk_box_pack_start(GTK_BOX(hbox), pixmap, FALSE, FALSE,0);

	label = gtk_label_new(title);
	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE , FALSE,2);

	return button; 
}
/****************************************************************************/
/* this function copies the name of the selected referential in the 
 * global variable */
void nc81_ref_select(gchar* ref_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_ref_name == ref_name)
	/* nothing to do if names are NULL */
	if (nc81_ref_name == NULL && ref_name == NULL) 
		return;
	/* nothing to do if names are the same */
	if (nc81_ref_name != NULL && ref_name != NULL && 
			strcmp(nc81_ref_name, ref_name) == 0)
		return;
	/* copy the referential name in the global variable */
	if (nc81_ref_name != NULL)
		g_free(nc81_ref_name);
	nc81_ref_name = (ref_name == NULL)?NULL:g_strdup(ref_name);
	/* erase task and job */
	nc81_task_select(NULL);
}

/****************************************************************************/
/* this function copies the name of the selected day in the 
 * global variable */
void nc81_day_select(gchar* day_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_day_name == day_name)
		return;
	/* nothing to do if names are NULL */
	if (nc81_day_name == NULL && day_name == NULL) 
		return;
	/* nothing to do if names are the same */
	if (nc81_day_name != NULL && day_name != NULL && 
			strcmp(nc81_day_name, day_name) == 0)
		return;
	/* copy the day name in the global variable */
	if (nc81_day_name != NULL)
		g_free(nc81_day_name);
	nc81_day_name = (day_name == NULL)?NULL:g_strdup(day_name);
	/* erase task and job */
	nc81_task_select(NULL);
}
/****************************************************************************/
/* this function copies the name of the selected referential task in the 
 * global variable */
void nc81_task_select(gchar* task_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_task_name == task_name)
		return;
	/* nothing to do if names are NULL */
	if (nc81_task_name == NULL && task_name == NULL) 
		return;
	/* nothing to do if names are the same */
	if (nc81_task_name != NULL && task_name != NULL && 
			strcmp(nc81_task_name, task_name) == 0)
		return;
	/* copy the task name in the global variable */
	if (nc81_task_name != NULL)
		g_free(nc81_task_name);
	nc81_task_name = (task_name == NULL)?NULL:g_strdup(task_name);
	/* erase job name */
	nc81_job_select(NULL);
}
/****************************************************************************/
/* this function copies the name of the selected referential job in the 
 * global variable */
void nc81_job_select(gchar* job_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_job_name == job_name)
		return;
	/* nothing to do if names are NULL */
	if (nc81_job_name == NULL && job_name == NULL) 
		return;
	/* nothing to do if names are the same */
	if (nc81_job_name != NULL && job_name != NULL && 
			strcmp(nc81_job_name, job_name) == 0)
		return;
	/* copy the job name in the global variable */
	if (nc81_job_name != NULL)
		g_free(nc81_job_name);
	nc81_job_name = (job_name == NULL)?NULL:g_strdup(job_name);
}
/****************************************************************************/
/* this function copies the name of the selected target in the 
 * global variable */
void nc81_target_select(gchar* target_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_target_name == target_name)
		return;
	/* copy the target name in the global variable */
	if (nc81_target_name != NULL)
		g_free(nc81_target_name);
	nc81_target_name = (target_name == NULL)?NULL:g_strdup(target_name);
	nc81_queue_select(NULL);
}
/****************************************************************************/
/* this function copies the name of the selected queue in the 
 * global variable */
void nc81_queue_select(gchar* queue_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_queue_name == queue_name)
		return;
	/* copy the queue name in the global variable */
	if (nc81_queue_name != NULL)
		g_free(nc81_queue_name);
	nc81_queue_name = (queue_name == NULL)?NULL:g_strdup(queue_name);
}
/****************************************************************************/
/* this function copies the name of the selected user in the 
 * global variable */
void nc81_user_select(gchar* user_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_user_name == user_name)
		return;
	/* copy the user name in the global variable */
	if (nc81_user_name != NULL)
		g_free(nc81_user_name);
	nc81_user_name = (user_name == NULL)?NULL:g_strdup(user_name);
}
/****************************************************************************/
/* this function copies the name of the selected rule in the 
 * global variable */
void nc81_rule_select(gchar* rule_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_rule_name == rule_name)
		return;
	/* copy the rule name in the global variable */
	if (nc81_rule_name != NULL)
		g_free(nc81_rule_name);
	nc81_rule_name = (rule_name == NULL)?NULL:g_strdup(rule_name);
}
/****************************************************************************/
/* this function copies the name of the selected variable in the 
 * global variable */
void nc81_variable_select(gchar* variable_name)
{
	/* nothing to do if pointers are the same */
	if (nc81_variable_name == variable_name)
		return;
	/* copy the variable name in the global variable */
	if (nc81_variable_name != NULL)
		g_free(nc81_variable_name);
	nc81_variable_name = (variable_name == NULL)?NULL:g_strdup(variable_name);
}
/****************************************************************************/
void nc81_set_user(gchar* value)
{
	/* nothing to do if pointers are the same */
	if (nc81_config_user == value)
		return;
	/* copy the value in the global variable */
	if (nc81_config_user != NULL)
		g_free(nc81_config_user);
	nc81_config_user = (value == NULL)?NULL:g_strdup(value);
	/* close the previous connection */
	nc81_command_close_connection();
}
/****************************************************************************/
void nc81_set_password(gchar* value)
{
	/* nothing to do if pointers are the same */
	if (nc81_config_password == value)
		return;
	/* copy the value in the global variable */
	if (nc81_config_password != NULL)
		g_free(nc81_config_password);
	nc81_config_password = (value == NULL)?NULL:g_strdup(value);
	/* close the previous connection */
	nc81_command_close_connection();
}
/****************************************************************************/
void nc81_set_server(gchar* value)
{
	/* nothing to do if pointers are the same */
	if (nc81_config_server == value)
		return;
	/* copy the value in the global variable */
	if (nc81_config_server != NULL)
		g_free(nc81_config_server);
	nc81_config_server = (value == NULL)?NULL:g_strdup(value);
	/* close the previous connection */
	nc81_command_close_connection();
}
/****************************************************************************/
void nc81_set_ip_port(gchar* value)
{
	/* nothing to do if pointers are the same */
	if (nc81_config_ip_port == value)
		return;
	/* copy the value in the global variable */
	if (nc81_config_ip_port != NULL)
		g_free(nc81_config_ip_port);
	nc81_config_ip_port = (value == NULL)?NULL:g_strdup(value);
	/* close the previous connection */
	nc81_command_close_connection();
}
/****************************************************************************/
GtkWidget* nc81_field_with_icon_new_pack(gchar* label, 
		GtkWidget* widget_to_insert, GtkBox* box, nc81_icon_type icon)
{
	GtkWidget* hbox;
	GtkWidget* widget;
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, TRUE, 1);
	/* create icon */
	if (icon != nc81_icon_none)
	{
		widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[icon], 
			nc81_icon_bitmap_array[icon]);
		gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	}
	/* create label */
	widget = gtk_label_new(label);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0.5);
	/* insert widget */
	gtk_box_pack_end(GTK_BOX(hbox), widget_to_insert, TRUE, TRUE, 1);

	return hbox;
}
/****************************************************************************/
GtkWidget* nc81_field_new_pack(gchar* label, GtkWidget* widget, GtkBox* box)
{
	return nc81_field_with_icon_new_pack(label, widget, 
			box, nc81_icon_none);
}
/****************************************************************************/
void nc81_paned_new(GtkWidget* vbox1, GtkWidget* vbox2, GtkWidget* container)
{
	GtkWidget* note_book;
	GtkWidget* hpaned;
	if (small)
	{	
		note_book = gtk_notebook_new();
		gtk_box_pack_start(GTK_BOX(container), 
				note_book, TRUE, TRUE, 1);
		gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox1,
			gtk_label_new("List"));
		gtk_container_set_border_width(GTK_CONTAINER(vbox1), 5);
		gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox2,
			gtk_label_new("Properties"));
		gtk_container_set_border_width(GTK_CONTAINER(vbox2), 5);
	} else {
		hpaned = gtk_hpaned_new();
		gtk_box_pack_start(GTK_BOX(container), hpaned, TRUE, TRUE, 1);
		gtk_paned_pack1(GTK_PANED(hpaned), vbox1, TRUE, TRUE);
		gtk_paned_pack2(GTK_PANED(hpaned), vbox2, TRUE, TRUE);
	}
}
/****************************************************************************/
