/* file icon.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include "icon.h"

static char * referential_xpm[] = {
"16 16 5 1",
" 	s None	c None",
".	c blue",
"X	c white",
"o	c #000080",
"O	c #969696",
"       .        ",
"      ...       ",
"     .....      ",
"    .......     ",
"   ....X....    ",
"  ....XXX....   ",
" oo..XXX......  ",
" ooo..X.......o ",
"  ooo........oO ",
"   ooo......oXO ",
"    ooo....oXXo ",
"     ooo..oXXo  ",
"      ooooXXo   ",
"       oooXo    ",
"        ooo     ",
"                "};

static char * task_xpm[] = {
"16 16 5 1",
"  s None c None",
". c #ff0000",
"# c #0000ff",
"a c #ff8000",
"b c #800000",
"                ",
"    bbbbb       ",
"   baaabb       ",
"  baaabbb       ",
" bbbbbb####     ",
" b...bb####     ",
" b...bbb ##     ",
" b...bb  ##bbbb ",
" bbbbb   ##aabb ",
"        baaabbb ",
"       bbbbbbbb ",
"       b...bbbb ",
"       b...bbb  ",
"       b...bb   ",
"       bbbbb    ",
"                "};

static char * job_xpm[] = {
"16 16 4 1",
"  s None c None",
". c #0000ff",
"# c #00ffff",
"a c #000080",
"                ",
"                ",
"                ",
"      aaaaaaaa  ",
"     a######aa  ",
"    a######aaa  ",
"   a######aaaa  ",
"  aaaaaaaaaaaa  ",
"  a.......aaaa  ",
"  a.......aaaa  ",
"  a.......aaaa  ",
"  a.......aaa   ",
"  a.......aa    ",
"  a.......a     ",
"  aaaaaaaa      ",
"                "};

static char * day_xpm[] = {
"16 16 7 1",
"  s None c None",
". c #bfc000",
"# c #ffff00",
"a c #000000",
"b c #c8c8c8",
"c c #ffffff",
"d c #c4c500",
"  aa         aa ",
" adda baaab adda",
" a##daa###aad##a",
" adda###a.##adda",
"  aa#.aacaa.#aa ",
"  a#.accacca.#a ",
" ba#acccaccca.#a",
" a#.acccaccca.#a",
" a#accccaaacca#a",
" a#.accccccca.#a",
" aa#accccccca#ab",
" ba#.accccca.#a ",
"  ba#.aacaa.#ab ",
" ba#a##.a.##a#ab",
" a#abaa###aaba#a",
" aab  baaab  baa"};

static char * monitor_xpm[] = {
"16 16 7 1",
"  s None c None",
". c #646464",
"# c #000000",
"a c #0000ff",
"b c #00ffff",
"c c #ffffff",
"d c #969696",
"                ",
"                ",
"      # # # #   ",
"    # ####### # ",
"  # ###aa###### ",
" # #daaaaaad####",
"  #dcabaaaacd## ",
" #dcaaa##aaacc##",
" #ccaaa##aaacc# ",
" .#ccaaaaaacd#. ",
"  .##aaaaaad##. ",
"   ..#######..  ",
"     .......    ",
"                ",
"                ",
"                "};
static char * diogene87_xpm[] = {
"39 64 17 1",
" 	c None",
".	c #030703",
"+	c #0D270D",
"@	c #143C14",
"#	c #215E1F",
"$	c #338128",
"%	c #318E30",
"&	c #309B3A",
"*	c #49A83B",
"=	c #0B1D09",
"-	c #286F24",
";	c #42A53A",
">	c #1B4D1A",
",	c #061306",
"'	c #2E9635",
")	c #123411",
"!	c #3EA23A",
"                                       ",
"                                       ",
"              .....                    ",
"              .$-$...                  ",
"             .$$$!&$..                 ",
"             .$>$&&&%..                ",
"            ..$+$!&&&#%.               ",
"            .$)@>;;&'-'.               ",
"            .$+$=%!&'$&'.              ",
"            .$=&>%&'%%&&.              ",
"            .%=&-%&'%%&-$.             ",
"      ....  .$+''#&&$%;;%.             ",
"      .#$.. .#)'&)'&'$!*%-.            ",
"      .-$'.  .@''='''$&*%$.            ",
"      ..%&.. .>%')'&&@&&'%.            ",
"        .'&...>%&>%&&$-''%$.           ",
"        ..&-.-#'&-'&&&=-&$$.           ",
"         .&>-'-%&-$;;&&#%$$.           ",
"         .'&'&-%&-%;*&&&+%$.           ",
"         .&''&-%'-$;;!&&=%%.           ",
"        .-'$-&-%'%$&&;&'+%$.  ...      ",
"        .-'$#%%$''>'&;&&+%$. .&%.,     ",
"        .-%%>'%%''+'&;&&+$$. .'$$.     ",
"        .-''@&'-%&>%&&&&)%%. .'%%..    ",
"        .-&'+&&@%&&#'&&&)&'...!;$$.    ",
"        .-%&+''=$&&+$&&&)'&'>&!&$$.    ",
"        .-%;+''+$&&=$;&&@'&'&!!'$-.    ",
"        .-%&='&=$&')#!&!@'&&&&%$-..    ",
"        .-%!+$')'*&>>!;&)'&'%$$#..     ",
"        .-%&+$&@'*&#>&!&@'&%$-...      ",
"         .$&@-'$&*&#@&&&+%$-...        ",
"         .$&)$-'&*&#)&&&+$$..          ",
"         .$'+%#'&*&#@&&&=$..           ",
"         ..$='+&;*&>@!&&>$.            ",
"         ..$@&)';*&@>&&&-$.            ",
"          .%#&@!*;&)#;!&$$.            ",
"          .$$&-!;!&+-**!%$.            ",
"          ..''$!;!'=$**!%-.            ",
"           .'$';;!&=%!;;%-.            ",
"           .&#'!!&'+'&*;%-..           ",
"           .'>'!;!'+'&&;%#-.           ",
"     ...  .)&@'!;!&=$&&'%#-.  ...      ",
"    ..$.  .@&>%!;!&+-&&'%--.  .&.      ",
"    .$$$..$>&#%&!;&>@''!$#$.  .&.,     ",
"     .'&&'')''-&&;&$=&!!$-$.. .&..     ",
"     .$'&&&+'')'&;&'+'!!$-$$. .&-.     ",
"      ..'&'+&'#%'&&'$%!!$$%-...&-.     ",
"       .$&'+&&'@%!&'%-**-%%------.     ",
"       .$&'+&&&=$&&'%#**#!%%$--&#.     ",
"       ..''+&&&-)&;'%#!&@$!&&''$..     ",
"        .%'+;&&'+%*&$$'&+%*;&'$>.      ",
"        .%'+!;&&>%;%-$&&=%**%$#..      ",
"        .$%+&&&'$'&%=%&&=$!!$..        ",
"        .$%+&&&'%%%+#%&&+$'$..         ",
"        .$%+&&&&%%%=$%&&=$%%.          ",
"        .$%=&&&&$%$-$&&&=$'%.          ",
"        .-%)&&&&>%#$%&&&=$&'$.         ",
"         .%#&&&'+'-$'&&&=$&&%$.        ",
"         .%'&&&$+&$$%&&'=%&&'-.        ",
"         .$'&&&@#&%-&&&'+'&&&'$.       ",
"         .$&&&')''%='&&'''&'&&%.       ",
"        .-$&&&&-&&%#'&&&'&&&&&&&.      ",
"       ...........................     ",
"                                       "};
static char ** xpm_array[] = {
	referential_xpm,
	day_xpm,
	task_xpm,
	job_xpm,
	monitor_xpm,
	diogene87_xpm
};

GdkPixmap *nc81_icon_pixmap_array[NC81_ICON_TYPE_COUNT + 1];
GdkBitmap *nc81_icon_bitmap_array[NC81_ICON_TYPE_COUNT + 1];

void nc81_icon_load(GtkWidget *main_window)
{
	GdkColor transparent;
	int i;
	for (i = 0; i < NC81_ICON_TYPE_COUNT; i++) 
	{
		nc81_icon_pixmap_array[i] = gdk_pixmap_create_from_xpm_d( 
					main_window->window,
					&nc81_icon_bitmap_array[i], 
					&transparent, 
					xpm_array[i]);
		assert(nc81_icon_pixmap_array[0]!=NULL);
		assert(nc81_icon_pixmap_array[i]!=NULL);
		assert(nc81_icon_bitmap_array[i]!=NULL);
	}

}
