/* file monitor.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

#define STR_ALL "<< all >>"

#define INDEX_RUNNING 1
#define INDEX_WAITING 2 
#define INDEX_TERMINATED 3
#define INDEX_HOLDED 4
#define INDEX_ERROR 5
#define INDEX_CANCEL 6

static guint update_callback_id = -1;
static char* state_array[] = {
	"Total",
	"Running",
	"Waiting",
	"Terminated",
	"Holded",
	"Error",
	"Cancel",
	NULL
};

static char* state_option_array[] = {
	".",
	"r", /* running */
	"w", /* waiting */
	"t", /* terminated */
	"h", /* holded */
	"e", /* error */
	"c", /* canceled */
	NULL
};

static GtkWidget* state_entry_array[7];
static GtkWidget* state_check_array[7];

static GtkWidget *day_combo;
static GtkWidget *view_job_button;
static GtkWidget *list;
static GtkWidget *field_interval;
static GList *combo_items;
static gboolean view_jobs;
static gint selected_row;

/****************************************************************************/
/* this function is called when a task or a job is selected in the list */
void list_select_child(GtkWidget* gtk_clist, gint row, 
		gint column, GdkEvent *event, gpointer data)
{
	selected_row = row;
	g_print("row%i\n", selected_row);
}
/****************************************************************************/
/* this function is called when a task or a job is unselected in the list */
void list_unselect_child(GtkWidget* gtk_clist, gint row, 
		gint column, GdkEvent *event, gpointer data)
{
	selected_row = -1;
	g_print("row%i\n", selected_row);
}
/****************************************************************************/
/* this function is called for every line to add in the day list */
static void day_list_callback(gchar** fields, void* data)
{
	gchar* str;
	str = g_strjoinv(" ", fields);
	combo_items = g_list_append(combo_items, str);
}
/****************************************************************************/
static void day_list_update()
{
	gchar* str;
	combo_items = NULL;
	/* add `all' at the begining of the list */
	str = g_strdup(STR_ALL);
	combo_items = g_list_append(combo_items, str);
	nc81_command_send("xdl", day_list_callback, NULL, NULL, NULL,NULL);
	gtk_combo_set_popdown_strings(GTK_COMBO(day_combo), combo_items);
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);
}
/****************************************************************************/
static void counters_update_callback(gchar** fields, void* data)
{
	int i;
	for (i = 0; state_array[i] != NULL; i++)
		if (strcmp(state_array[i], fields[0]) == 0)
			gtk_entry_set_text(GTK_ENTRY(state_entry_array[i]), 
					fields[1]);
}
/****************************************************************************/
static void list_update_callback(gchar** fields, void* data)
{
	gchar* values[5];

	values[0] = fields[0]; /* S */
	values[1] = fields[1]; /* day */
	values[2] = fields[2]; /* task */
	values[3] = "";
	values[4] = "";
	if (view_jobs == TRUE) {
		values[3] = fields[3]; 
		values[4] = fields[4]; 
	} else {
		values[4] = fields[3]; 
	}
	gtk_clist_append(GTK_CLIST(list), values);
}
/****************************************************************************/
static void list_update(void)
{
	int i;
	char flag[10];
	gchar* day;
	gchar* cmd;
	/* get day name */
	day = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(day_combo)->entry));
	if (strcmp(day, STR_ALL) == 0)
		day = NULL;
	/* view task/job */
	view_jobs = gtk_toggle_button_get_active(
			GTK_TOGGLE_BUTTON(view_job_button));
	/* update counters */
	if (view_jobs == TRUE)
		cmd = g_strdup_printf("ejm %s", (day==NULL) ? "" : day);
	else
		cmd = g_strdup_printf("etm %s", (day==NULL) ? "" : day);
	nc81_command_send(cmd, counters_update_callback, NULL, NULL, NULL,NULL);
	g_free(cmd);

	/* create flags */
	strcpy(flag, "");
	for (i = 1; state_option_array[i] != NULL; i++)
	{
		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
						state_check_array[i])) == TRUE)
			strcat(flag, state_option_array[i]);
	}
	/* create command */
	if (view_jobs == TRUE)
		cmd = g_strdup_printf("ejm -%s %s", flag, (day==NULL)?"":day);
	else
		cmd = g_strdup_printf("etm -%s %s", flag, (day==NULL)?"":day);
	/* update list */
	gtk_clist_freeze(GTK_CLIST(list));
	gtk_clist_clear(GTK_CLIST(list));
	gtk_clist_set_column_visibility(GTK_CLIST(list), 3, view_jobs);
	if (strcmp(flag,"") != 0)
		nc81_command_send(cmd, list_update_callback, 
				NULL, NULL, NULL,NULL);
	gtk_clist_thaw(GTK_CLIST(list));
	g_free(cmd);
	selected_row = -1;
	return;
}
/****************************************************************************/
static gboolean update_callback(gpointer data)
{
	if (update_callback_id == -1)
		return TRUE;
	list_update();
	return TRUE;
}
/****************************************************************************/
static void update_callback_remove(void)
{
	if (update_callback_id != -1)
	{
		gtk_timeout_remove(update_callback_id);
		update_callback_id = -1;
	}
}
/****************************************************************************/
static void update_callback_add(void)
{
	guint interval;
	interval = atoi(gtk_entry_get_text(GTK_ENTRY(field_interval))) * 1000;
	g_print("timer interval :%i\n", interval);
	update_callback_remove();
	if (interval <= 0)
		return;
	update_callback_id = gtk_timeout_add(interval, update_callback, NULL);
}

/****************************************************************************/
static gint button_update_clicked(GtkWidget *widget, gpointer data)
{
	list_update();
	return 0;
}

/****************************************************************************/
static gint button_select_clicked(GtkWidget *widget, gpointer data)
{
	gchar* name;
	if (selected_row == -1)
		return 0;
	/* select day */
	gtk_clist_get_text(GTK_CLIST(list), selected_row, 1 ,&name);
	nc81_day_select(name);
	/* select task */
	gtk_clist_get_text(GTK_CLIST(list), selected_row, 2 ,&name);
	nc81_task_select(name);

	if (view_jobs == TRUE)
	{
		/* select job */
		gtk_clist_get_text(GTK_CLIST(list), selected_row, 3 ,&name);
		nc81_job_select(name);
		nc81_main_load(nc81_form_exploitation_job_view);
	}
	else
	{
		nc81_main_load(nc81_form_exploitation_task_view);
	}
	return 0;
}
/****************************************************************************/
/* this function creates a widget to follow job/task execution */
GtkWidget* nc81_monitor_new(void)
{
	GtkWidget *container;
	GtkWidget *notebook;
	GtkWidget *widget;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *bbox;
	GtkWidget *table;
	GtkWidget *scrolled;
	GtkObject *adj;
	gchar* titles[5];
	GSList *group;
	int i;

	container = gtk_vbox_new(FALSE, 2);

	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(container), notebook, TRUE, TRUE, 0);
	
	/* selection panel */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			vbox, gtk_label_new("Selection"));

	/* task/job */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);

	widget = gtk_radio_button_new_with_label(NULL, "Tasks");
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(widget));
	widget = gtk_radio_button_new_with_label(group, "Jobs");
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0);
	view_job_button = widget;

	
	/* day combo */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Day");
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0);
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0.5);

	widget = gtk_combo_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0);
	gtk_combo_set_value_in_list(GTK_COMBO(widget), 1, 0); 
	day_combo = widget;

	
	/* udpate interval */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);

	widget = gtk_label_new("Update interval");
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0.5);

	adj = gtk_adjustment_new(5, 0, 120, 5, 10, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_signal_connect(GTK_OBJECT(widget),
			"changed",
			GTK_SIGNAL_FUNC(update_callback_add),
			NULL);
	field_interval = widget;

	/* state frame */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			vbox, gtk_label_new("Filters"));
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	table = gtk_table_new(5, 1, FALSE);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled), table);

	for (i = 0; state_array[i] != NULL; i++)
	{
		if (i == 0)
			widget=gtk_label_new(state_array[i]);
		else
			widget=gtk_check_button_new_with_label(state_array[i]);
		gtk_table_attach_defaults(GTK_TABLE(table), widget,
				0, 1, i, i+ 1);
		state_check_array[i] = widget;
		widget = gtk_entry_new();
		gtk_widget_set_usize(GTK_WIDGET(widget), 40, 0);
		gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
		gtk_table_attach_defaults(GTK_TABLE(table), widget, 
				1, 2, i, i+ 1);
		state_entry_array[i] = widget;
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
				state_check_array[INDEX_RUNNING]), TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
				state_check_array[INDEX_ERROR]), TRUE);

	widget = gtk_button_new_with_label("Update");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 1);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_update_clicked), NULL);

	/* list frame */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			vbox, gtk_label_new("List"));
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	titles[0] = "S";
	titles[1] = "Referential";
	titles[2] = "Task";
	titles[3] = "Job";
	titles[4] = "State";
	widget = gtk_clist_new_with_titles(5,titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 10);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 150);
	gtk_clist_set_column_width(GTK_CLIST(widget), 2, 90);
	gtk_clist_set_column_width(GTK_CLIST(widget), 3, 90);
	gtk_clist_set_column_width(GTK_CLIST(widget), 4, 90);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "select_row",
			list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(widget), "unselect_row",
			list_unselect_child, NULL);
	list = widget;


	/* button box */
	bbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);

	widget = gtk_button_new_with_label("Select");
	gtk_container_add(GTK_CONTAINER(bbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_select_clicked), NULL);

	widget = gtk_button_new_with_label("Update");
	gtk_container_add(GTK_CONTAINER(bbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_update_clicked), NULL);

	day_list_update();
	list_update();

	update_callback_add();
	/* remove the callback function when the container is destroyed */
	gtk_signal_connect(GTK_OBJECT(container),
			"destroy",
			GTK_SIGNAL_FUNC(update_callback_remove),
			NULL);
	
	return container;
}
/****************************************************************************/
