/* file queue_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget *field_name;
static GtkWidget *field_description;
static GtkWidget *field_max;
static GtkWidget *field_opened;

/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Queue Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);
	
	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_description), fields[1]);

	if (strcmp(fields[0], "Max") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_max), fields[1]);

	if (strcmp(fields[0], "Opened") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_opened), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);
}
/****************************************************************************/
/* this function updates the queue fields */
static void nc81_queue_edit_update(void)
{
	if (nc81_queue_name == NULL)
		return;
	nc81_command_send("qi \"%T\" \"%Q\"", field_fill_callback, 
			NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* queue_name;
	gboolean changed;
	gboolean opened;
	gchar* value;

	/* queue name */
	queue_name = gtk_entry_get_text(GTK_ENTRY(field_name));
	if (nc81_queue_name == NULL)
	{
		/* create a new queue */
		nc81_command_send( "qa \"%T\" \"%x\"",  
			field_name_valid_callback,
			NULL, queue_name, NULL, NULL);
		nc81_queue_select(queue_name);
	}
	else
	{
		changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
					GTK_OBJECT(field_name)));
		if (changed == TRUE)
		{
			/* change the name of the target */
			nc81_command_send("qsn \"%T\" \"%Q\" \"%x\"", 
				field_name_valid_callback,
				NULL, queue_name, NULL, NULL);
			nc81_queue_select(queue_name);
		}
	}
	/* description */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_description)));
	if (changed == TRUE)
	{
		value = gtk_entry_get_text(GTK_ENTRY(field_description));
		nc81_command_send("qsd \"%T\" \"%Q\" \"%x\"", 
			field_name_valid_callback,
			NULL, value, NULL, NULL);
	}

	/* max */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_max)));
	if (changed == TRUE)
	{
		value = gtk_entry_get_text(GTK_ENTRY(field_max));
		nc81_command_send("qsm \"%T\" \"%Q\" \"%x\"", 
			field_name_valid_callback,
			NULL, value, NULL, NULL);
	}
	/* opened property */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_opened)));
	if (changed == TRUE)
	{
		opened = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_opened));
		nc81_command_send("qso \"%T\" \"%Q\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(opened == TRUE)?"true":"false",
			NULL,NULL);

	}

	/* TODO do not close if error */
	nc81_main_load(nc81_form_queue_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_queue_view);
	return 0;
}
/****************************************************************************/
static void nc81_field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}

/****************************************************************************/
/* this function creates a widget which edit or add a queue*/
GtkWidget* nc81_queue_edit_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	vbox = gtk_vbox_new(FALSE, 2);

	/* queue name field */
	field_name = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_name), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack(
			(nc81_queue_name==NULL)? "New queue": "Queue",
			field_name, GTK_BOX(vbox));

	/* description field */
	field_description = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_description), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Description", field_description, GTK_BOX(vbox));

	/* max field */
	field_max = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_max), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Maximum", field_max, GTK_BOX(vbox));

	/* field opened */
	widget = gtk_check_button_new_with_label("Opened");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);
	field_opened = widget;

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	nc81_queue_edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_name), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_description), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_max), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_opened), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	return vbox;
}
/****************************************************************************/
