/* file queue_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* queue_list;
static GtkWidget* field_name;
static GtkWidget* field_description;
static GtkWidget* field_target;
static GtkWidget* field_opened;
static GtkWidget* field_max;

static GtkWidget* button_edit;
static GtkWidget* button_delete;
static GtkWidget* button_target;

/****************************************************************************/
/* this callback function is called to fill the field values */
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]);*/

	if (strcmp(fields[0], "Queue Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);

	if (strcmp(fields[0], "Target Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_target), fields[1]);

	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_description), fields[1]);

	if (strcmp(fields[0], "Max") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_max), fields[1]);

	if (strcmp(fields[0], "Opened") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_opened), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);
}


/****************************************************************************/
/* this function intializes the queue fields
 * and copies the name of the selected queue in the global queue
 */
static void nc81_queue_display(gchar* queue_name)
{
	gboolean enabled;

	nc81_queue_select(queue_name);

	if (nc81_queue_name == NULL)
	{
		/* no queue selected : blank fields */
		gtk_entry_set_text(GTK_ENTRY(field_name), "");
		gtk_entry_set_text(GTK_ENTRY(field_target), "");
		gtk_entry_set_text(GTK_ENTRY(field_description), "");
		gtk_entry_set_text(GTK_ENTRY(field_max), "");
		/*TODO*/
		enabled = FALSE;
	}
	else
	{
		nc81_command_send("qi \"%T\" \"%Q\"", field_fill_callback,
				NULL,NULL,NULL,NULL);
		enabled = TRUE;
	}
	gtk_widget_set_sensitive(button_edit, enabled);
	gtk_widget_set_sensitive(button_delete, enabled);
	gtk_widget_set_sensitive(button_target, enabled);

}
/****************************************************************************/
/* this function is called when a queue is selected */
static void nc81_queue_list_select_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	gchar* str;
	GtkWidget* label;
	label = g_list_nth_data(
			gtk_container_children(GTK_CONTAINER(list_item) ), 0);
	gtk_label_get(GTK_LABEL(label), &str);
	nc81_queue_display(str);
	/* TODO g_free(str)? */
}
/****************************************************************************/
/* this function is called when a queue is unselected */
static void nc81_queue_list_unselect_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	/* do not nc81_queue_display(NULL)! */
}
/****************************************************************************/
/* this function is called for every line to add in the queue list */
static void list_fill_callback(gchar** fields, void* data)
{
	gchar* str;
	GList *item_list;
	GtkWidget *item;
	str = g_strjoinv(" ", fields);
	/* create a list item */
	item = gtk_list_item_new_with_label(str);
	gtk_widget_show(GTK_WIDGET(item));
	/* add this item in the list */
	item_list = g_list_append(NULL, item); 
	gtk_list_append_items(GTK_LIST(queue_list), item_list);
	/* select the queue */
	if (nc81_queue_name != NULL && strcmp(fields[0], nc81_queue_name) == 0)
		gtk_list_select_child(GTK_LIST(queue_list), item);
	g_free(str);
}

/****************************************************************************/
/* this function updates the queue list content, and
 * the queue fields */
static void nc81_queue_view_update(void)
{
	gtk_list_clear_items(GTK_LIST(queue_list),0,-1);
	nc81_queue_display(nc81_queue_name);
	nc81_command_send("ql \"%T\"", list_fill_callback, 
			NULL, NULL, NULL, NULL);
}

/****************************************************************************/
static gint button_edit_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_queue_edit);
	return 0;
}
/****************************************************************************/
static gint button_new_clicked(GtkWidget *widget, gpointer data)
{
	nc81_queue_select(NULL);
	nc81_main_load(nc81_form_queue_edit);
	return 0;
}
/****************************************************************************/
static gint button_delete_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_queue_delete);
	return 0;
}
/****************************************************************************/
static gint button_target_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_target_view);
	return 0;
}
/****************************************************************************/
/*
 * this function creates a widget which contains a list of all
 * queues, and the queue fields
 */
GtkWidget* nc81_queue_view_new(void)
{
	GtkWidget *container;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;

	container = gtk_vbox_new(FALSE, 2);
	vbox1 = gtk_vbox_new(FALSE, 1);
	vbox2 = gtk_vbox_new(FALSE, 2);
	nc81_paned_new(vbox1, vbox2, container);


	/* target name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_target_name);
	nc81_field_with_icon_new_pack("Target", widget, 
			GTK_BOX(vbox1), nc81_icon_none);
	/* TODO add icon of target */

	/* construct the queue list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 1);

	/* TODO change icon */
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_day], 
			nc81_icon_bitmap_array[nc81_icon_day]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_label_new("Queue list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_list_new();
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	queue_list = widget;

	gtk_signal_connect(GTK_OBJECT(queue_list),
			"select_child",
			nc81_queue_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(queue_list),
			"unselect_child",
			nc81_queue_list_unselect_child, NULL);

	/* construct queue fields */
	field_name = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_name), FALSE);
	nc81_field_new_pack("Queue name", field_name, GTK_BOX(vbox2));

	field_description = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_description), FALSE);
	nc81_field_new_pack("Description", field_description, GTK_BOX(vbox2));

	field_target = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_target), FALSE);
	nc81_field_new_pack("Target", field_target, GTK_BOX(vbox2));

	widget = gtk_label_new("Maximum number of running jobs");
	gtk_box_pack_start(GTK_BOX(vbox2), widget, FALSE, TRUE, 1);
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(vbox2), widget, FALSE, TRUE, 1);
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	field_max = widget;

	widget = gtk_check_button_new_with_label("Opened");
	gtk_box_pack_start(GTK_BOX(vbox2), widget, FALSE, TRUE, 1);
	gtk_widget_set_sensitive(widget, FALSE);
	field_opened = widget;


	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Edit");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_edit_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_edit = menu_item;

	menu_item = gtk_menu_item_new_with_label("Add");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_new_clicked), NULL);

	menu_item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_delete_clicked), NULL);
	button_delete = menu_item;

	nc81_main_set_edit_menu(GTK_MENU(menu));

	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Target");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_target_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_target = menu_item;



	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_queue_view_update();

	return container;
}
/****************************************************************************/
