/* file ref_cond_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static char* tb_type[] = {
	"hmin",
	"hmax",
	"hlate",
	"file",
	"manual",
	"if_equal",
	"if_not_equal",
	NULL
};

static GtkWidget *notebook;
static GtkWidget *field_hmin_hour;
static GtkWidget *field_hmin_min;
static GtkWidget *field_hmin_delta;
static GtkWidget *field_hmax_hour;
static GtkWidget *field_hmax_min;
static GtkWidget *field_hmax_delta;
static GtkWidget *field_hlate_hour;
static GtkWidget *field_hlate_min;
static GtkWidget *field_hlate_delta;
static GtkWidget *field_message;
static GtkWidget *field_file;
static GtkWidget *field_ife_variable; /* if_equal - name */
static GtkWidget *field_ife_value; /* if_equal - value*/
static GtkWidget *field_ifn_variable; /* if_not_equal - name */
static GtkWidget *field_ifn_value; /* if_not_equal - value*/
static GList *combo_items;

/****************************************************************************/
static void valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar *arg,*str, *str2;
	gint h,m,d;

	switch (gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))
	{
		case 0: /* hmin */
			h=atoi(gtk_entry_get_text(GTK_ENTRY(field_hmin_hour)));
			m=atoi(gtk_entry_get_text(GTK_ENTRY(field_hmin_min)));
			d=atoi(gtk_entry_get_text(GTK_ENTRY(field_hmin_delta)));
			arg = g_strdup_printf("hmin \"%02i:%02i\" %i",h,m,d);
			break;
		case 1: /* hmax */
			h=atoi(gtk_entry_get_text(GTK_ENTRY(field_hmax_hour)));
			m=atoi(gtk_entry_get_text(GTK_ENTRY(field_hmax_min)));
			d=atoi(gtk_entry_get_text(GTK_ENTRY(field_hmax_delta)));
			arg = g_strdup_printf("hmax \"%02i:%02i\" %i",h,m,d);
			break;
		case 2: /* hlate */
			h=atoi(gtk_entry_get_text(GTK_ENTRY(field_hlate_hour)));
			m=atoi(gtk_entry_get_text(GTK_ENTRY(field_hlate_min)));
			d=atoi(gtk_entry_get_text(GTK_ENTRY(field_hlate_delta)));
			arg = g_strdup_printf("hlate \"%02i:%02i\" %i",h,m,d);
			break;

		case 3: /* file */
			str = gtk_entry_get_text(GTK_ENTRY(field_file));
			arg = g_strdup_printf("file \"%s\"", str);
			break;
		case 4: /* manual */
			str = gtk_entry_get_text(GTK_ENTRY(field_message));
			arg = g_strdup_printf("manual \"%s\"", str);
			break;
		case 5: /* if_equal */
			str = gtk_entry_get_text(GTK_ENTRY(
					GTK_COMBO(field_ife_variable)->entry));
			str2 = gtk_entry_get_text(GTK_ENTRY(field_ife_value));
			arg = g_strdup_printf("if_equal \"%s\" \"%s\"", 
					str, str2);
			break;
		case 6: /* if_not_equal */
			str = gtk_entry_get_text(GTK_ENTRY(
					GTK_COMBO(field_ifn_variable)->entry));
			str2 = gtk_entry_get_text(GTK_ENTRY(field_ifn_value));
			arg = g_strdup_printf("if_not_equal \"%s\" \"%s\"", 
					str, str2);
			break;

	}

	nc81_command_send("rtca \"%r\" \"%t\" %x", 
		valid_callback, NULL, arg, NULL, NULL);
	g_free(arg);

	nc81_main_load( nc81_form_referential_task_condition_view);
	return 0;
}
/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load( nc81_form_referential_task_condition_view);
	return 0;
}
/****************************************************************************/
static void combo_init_callback(gchar** fields, void* data)
{
	gchar* str;
	str = g_strjoinv(" ", fields);
	combo_items = g_list_append(combo_items, str);
}
/****************************************************************************/
static void init_fields(void)
{
	/* init variable list */
	combo_items = NULL;
	nc81_command_send("vl", combo_init_callback, NULL, NULL, NULL,NULL);
	if (combo_items != NULL)
	{
		gtk_combo_set_popdown_strings(GTK_COMBO(field_ife_variable), 
			combo_items);
		gtk_combo_set_popdown_strings(GTK_COMBO(field_ifn_variable), 
			combo_items);
	}
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);

}
/****************************************************************************/
static gint condition_type_clicked(GtkWidget *widget, gpointer data)
{
	int i;
	i = (gint)data;
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), i);
	return 0;
}
/****************************************************************************/
/* this function creates a widget to create a task condition */
GtkWidget* nc81_ref_task_cond_edit_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *menu_item;
	GtkWidget *menu;
	GtkWidget *page;
	GtkObject *adj;
	int i;

	assert(nc81_ref_name != NULL);
	vbox = gtk_vbox_new(FALSE, 1);

	/* task */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new("Task");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE,1);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_task_name);

	/* referential name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new("Referential");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);

	/* condition type */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new("Type");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_option_menu_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	menu = gtk_menu_new();

	for (i = 0; tb_type[i] != NULL; i++)
	{
		menu_item = gtk_menu_item_new_with_label(tb_type[i]);
		gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
				GTK_SIGNAL_FUNC(condition_type_clicked), 
				(gpointer)i);
		gtk_menu_append(GTK_MENU(menu), menu_item);
	}
	gtk_widget_show_all(menu);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(widget), menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 0);


	/* notebook */
	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(vbox), notebook, FALSE, FALSE,1);
	gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);

	/* hmin */
	page = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Min time");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	/* hour */
	adj = gtk_adjustment_new(0, 0, 23, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hmin_hour = widget;
	/* minute */
	adj = gtk_adjustment_new(0, 0, 59, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hmin_min = widget;
	/* delta */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Delta day");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(0, 0, 9, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hmin_delta = widget;

	/* hmax */
	page = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Max time");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	/* hour */
	adj = gtk_adjustment_new(0, 0, 23, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hmax_hour = widget;
	/* minute */
	adj = gtk_adjustment_new(0, 0, 59, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hmax_min = widget;
	/* delta */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Delta day");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(0, 0, 9, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hmax_delta = widget;

	/* hlate */
	page = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Time too late");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	/* hour */
	adj = gtk_adjustment_new(0, 0, 23, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hlate_hour = widget;
	/* minute */
	adj = gtk_adjustment_new(0, 0, 59, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hlate_min = widget;
	/* delta */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Delta day");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(0, 0, 9, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, FALSE, 1);
	field_hlate_delta = widget;


	/* file */
	page = gtk_hbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	widget = gtk_label_new("File");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, FALSE, 1);
	field_file = widget;

	/* message */
	page = gtk_hbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	widget = gtk_label_new("Message");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, FALSE, 1);
	field_message = widget;

	/* if_equal */
	page = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Variable");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	field_ife_variable = widget;

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Value");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	field_ife_value = widget;

	/* if_not_equal */
	page = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Variable");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	field_ifn_variable = widget;

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Value");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	field_ifn_value = widget;

	/* button box (ok/cancel) */
	hbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);

	/* button OK */
	widget = gtk_button_new_with_label("OK");
	gtk_container_add(GTK_CONTAINER(hbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);

	/* button Cancel */
	widget = gtk_button_new_with_label("Cancel");
	gtk_container_add(GTK_CONTAINER(hbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);


	/* variable value */
	init_fields();
	return vbox;
}
/****************************************************************************/
