/* file ref_dep_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static gboolean show_job_dep;
static int typestream = 0;
static int deptype = 0;
static GList *combo_items;
static GtkWidget *combo;

/****************************************************************************/
static void combo_init_callback(gchar** fields, void* data)
{
	gchar* str;
	str = g_strjoinv(" ", fields);
	combo_items = g_list_append(combo_items, str);
}
/****************************************************************************/
static void combo_init()
{
	combo_items = NULL;
	nc81_command_send(
			(show_job_dep == TRUE)?
				"rjl \"%r\" \"%t\"":
				"rtl \"%r\"",
			combo_init_callback, NULL, NULL, NULL,NULL);
	gtk_combo_set_popdown_strings(GTK_COMBO(combo), 
			combo_items);
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);
}
/****************************************************************************/
static void valid_callback(gchar** fields, void* data)
{

}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* upstream;
	gchar* downstream;
	gchar* type;
	gchar* name1;
	gchar* name2;

	name1 = (show_job_dep)?nc81_job_name:nc81_task_name;
	name2 = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(combo)->entry));
	if (typestream == 0)
	{
		upstream = name1;
		downstream = name2;
	} else {
		upstream = name2;
		downstream = name1;
	}

	switch (deptype)
	{
		case 0: type = "always"; break;
		case 1: type = "onsuccess"; break;
		case 2: type = "onerror"; break;
		default: type = ""; break;
	}

	nc81_command_send((show_job_dep==TRUE)?
			"rjda \"%r\" \"%t\" \"%x\" \"%y\" \"%z\"": 
			"rtda \"%r\" \"%x\" \"%y\" \"%z\"", 
		valid_callback, NULL, upstream, downstream, type);
	nc81_main_load((show_job_dep == TRUE)?
			nc81_form_referential_job_dep_view:
			nc81_form_referential_task_dep_view);
	return 0;
}
/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load((show_job_dep == TRUE)?
			nc81_form_referential_job_dep_view:
			nc81_form_referential_task_dep_view);
	return 0;
}
/****************************************************************************/
static gint typestream_clicked(GtkWidget *widget, gpointer data)
{
	typestream = (gint)data;
	return 0;
}
/****************************************************************************/
static gint deptype_clicked(GtkWidget *widget, gpointer data)
{
	deptype = (gint)data;
	return 0;
}
/****************************************************************************/
/* this function creates a widget to create a dependence */
static GtkWidget* nc81_ref_dep_edit_new(gboolean job_dep)
{
	GtkWidget *container;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *menu_item;
	GtkWidget *menu;

	show_job_dep = job_dep;
	container = gtk_vbox_new(FALSE,0);

	/* referential name */
	widget = gtk_entry_new();
	nc81_field_with_icon_new_pack("Referential", widget, 
			GTK_BOX(container), nc81_icon_referential);
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);

	/* task name */
	widget = gtk_entry_new();
	nc81_field_with_icon_new_pack("Task", widget, 
			GTK_BOX(container), nc81_icon_task);
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_task_name);

	/* job name */
	if (show_job_dep == TRUE)
	{
		widget = gtk_entry_new();
		nc81_field_with_icon_new_pack("Job", widget, 
			GTK_BOX(container), nc81_icon_job);
		gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
		gtk_entry_set_text(GTK_ENTRY(widget), nc81_job_name);
	}

	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, FALSE, 1);

	/* upstream/downstream */
	widget = gtk_option_menu_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Downstream");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(typestream_clicked), (gpointer)0);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Upstream");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(typestream_clicked), (gpointer)1);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	gtk_widget_show_all(menu);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(widget), menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 0);

	/* type always/onerror/onsuccess */ 
	widget = gtk_option_menu_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Always");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(deptype_clicked), (gpointer)0);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("On success");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(deptype_clicked), (gpointer)1);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("On error");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(deptype_clicked), (gpointer)2);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	gtk_widget_show_all(menu);
	gtk_option_menu_set_menu(GTK_OPTION_MENU(widget), menu);
	gtk_option_menu_set_history(GTK_OPTION_MENU(widget), 0);

	/* task/job list */
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new((show_job_dep)?"Linked job":"Linked task");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	combo = widget;

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, TRUE, 1);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	gtk_container_add(GTK_CONTAINER(hbox), widget);
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_container_add(GTK_CONTAINER(hbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);

	combo_init();

	return container;
}
/****************************************************************************/
GtkWidget* nc81_ref_task_dep_edit_new()
{
	return nc81_ref_dep_edit_new(FALSE);
}
/****************************************************************************/
GtkWidget* nc81_ref_job_dep_edit_new()
{
	return nc81_ref_dep_edit_new(TRUE);
}
/****************************************************************************/
