/* file ref_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget *field_name;
static GtkWidget *field_autopurged;
static GtkWidget *field_autoplanned;
/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Referential Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);

	if (strcmp(fields[0], "Auto Planned") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_autoplanned), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "Auto Purged") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_autopurged), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);
	
}
/****************************************************************************/
/* this function updates the referential fields */
static void nc81_ref_edit_update(void)
{
	if (nc81_ref_name == NULL)
		return;
	nc81_command_send("ri \"%r\"", field_fill_callback, 
			NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* ref_name;
	gboolean changed;
	gboolean autoplanned_changed;
	gboolean autopurged_changed;
	gboolean autoplanned;
	gboolean autopurged;

	/* referential name */
	ref_name = gtk_entry_get_text(GTK_ENTRY(field_name));
	if (nc81_ref_name == NULL)
	{
		/* create a new referential */
		nc81_command_send( "ra \"%x\"",  
			field_name_valid_callback,
			NULL, ref_name, NULL, NULL);
		nc81_ref_select(ref_name);
	}
	else
	{
		changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
					GTK_OBJECT(field_name)));
		if (changed == TRUE)
		{
			/* change the name of the referential */
			nc81_command_send("rsn \"%r\" \"%x\"", 
				field_name_valid_callback,
				NULL, ref_name, NULL, NULL);
			nc81_ref_select(ref_name);
		}
	}

	/* auto planed / auto purged */
	autoplanned_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_autoplanned)));
	autopurged_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_autopurged)));
	if (autoplanned_changed == TRUE || autopurged_changed == TRUE)
	{
		autoplanned = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_autoplanned));
		autopurged = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_autopurged));
		nc81_command_send("rsa \"%r\" \"%x\" \"%y\"", 
			field_name_valid_callback, NULL, 
			(autoplanned == TRUE)?"true":"false",
			(autopurged == TRUE)?"true":"false",
			NULL);
	}

	/* TODO do not close if error */
	nc81_main_load(nc81_form_referential_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_view);
	return 0;
}
/****************************************************************************/
static void nc81_field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}

/****************************************************************************/
/* this function creates a widget which edit or add a referential*/
GtkWidget* nc81_ref_edit_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	vbox = gtk_vbox_new(FALSE, 2);

	/* referential name field */
	field_name = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_name), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack(
			(nc81_ref_name==NULL)? "New referential": "Referential",
			field_name, GTK_BOX(vbox));

	/* auto planned field */
	widget = gtk_check_button_new_with_label("Auto planned");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_autoplanned = widget;

	/* auto purged field */
	widget = gtk_check_button_new_with_label("Auto purged");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_autopurged = widget;

	/* button box */
	widget = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 10);
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	nc81_ref_edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_name), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_autoplanned), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_autopurged), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	return vbox;
}
/****************************************************************************/
