/* file ref_job_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget *field_name;
static GtkWidget *field_description;
static GtkWidget *field_path;
static GtkWidget *field_argument;
static GtkWidget *field_wad; /* wait all dependencies */
static GtkWidget *field_oec; /* on error continue */
static GtkWidget *field_start_tempo;
static GtkWidget *field_end_tempo;


/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */
	if (strcmp(fields[0], "Job Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);

	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_description), fields[1]);
	
	if (strcmp(fields[0], "Wait All Dependencies") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_wad), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "On Error Continue") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_oec), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "Start Tempo") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_start_tempo), 
				fields[1]);

	if (strcmp(fields[0], "End Tempo") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_end_tempo), fields[1]);

	if (strcmp(fields[0], "Path") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_path), fields[1]);

	if (strcmp(fields[0], "Argument") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_argument), fields[1]);
}
/****************************************************************************/
/* this function updates the job fields */
static void nc81_ref_task_edit_update(void)
{
	if (nc81_job_name == NULL) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(field_wad),
				TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(field_oec),
				TRUE);
		return;
	}
	nc81_command_send("rji \"%r\" \"%t\" \"%j\"", 
			field_fill_callback, NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* job_name;
	gchar* value;
	gboolean changed;
	gboolean bool_value;
	gboolean start_tempo_changed;
	gboolean end_tempo_changed;
	gchar* start_tempo;
	gchar* end_tempo;

	/* job name */
	job_name = gtk_entry_get_text(GTK_ENTRY(field_name));
	if (nc81_job_name == NULL)
	{
		/* create a new job */
		nc81_command_send( "rja \"%r\" \"%t\" \"%x\"",  
			field_name_valid_callback,
			NULL, job_name, NULL,NULL);
		nc81_job_select(job_name);
	}
	else
	{
		changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
					GTK_OBJECT(field_name)));
		if (changed == TRUE)
		{
			/* change the name of the job */
			nc81_command_send("rjsn \"%r\" \"%t\" \"%j\" \"%x\"", 
				field_name_valid_callback,
				NULL, job_name, NULL,NULL);
			nc81_job_select(job_name);
		}
	}

	/* description field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_description)));
	if (changed == TRUE)
	{
		value = gtk_entry_get_text(GTK_ENTRY(field_description));
		nc81_command_send("rjsd \"%r\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			value,NULL,NULL);
	}

	/* path field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_path)));
	if (changed == TRUE)
	{
		value = gtk_entry_get_text(GTK_ENTRY(field_path));
		nc81_command_send("rjsp \"%r\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			value,NULL,NULL);
	}

	/* argument field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_argument)));
	if (changed == TRUE)
	{
		value = gtk_entry_get_text(GTK_ENTRY(field_argument));
		nc81_command_send("rjsa \"%r\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			value,NULL,NULL);
	}

	/* start/end tempo fields */
	start_tempo_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_start_tempo)));
	end_tempo_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_end_tempo)));
	if (start_tempo_changed || end_tempo_changed)
	{

		start_tempo = gtk_entry_get_text(GTK_ENTRY(field_start_tempo));
		end_tempo = gtk_entry_get_text(GTK_ENTRY(field_end_tempo));
		nc81_command_send("rjst \"%r\" \"%t\" \"%j\" \"%x\" \"%y\"", 
			field_name_valid_callback, NULL, 
			start_tempo, end_tempo, NULL);
	}


	/* wad field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_wad)));
	if (changed == TRUE)
	{
		bool_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_wad));
		nc81_command_send("rjswad \"%r\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(bool_value == TRUE)?"true":"false",
			NULL,NULL);
	}

	/* oec field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_oec)));
	if (changed == TRUE)
	{
		bool_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_oec));
		nc81_command_send("rjsoec \"%r\" \"%t\" \"%j\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(bool_value == TRUE)?"true":"false",
			NULL,NULL);
	}

	/* TODO do not close if error */
	nc81_main_load(nc81_form_referential_job_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_job_view);
	return 0;
}
/****************************************************************************/
static void nc81_field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}

/****************************************************************************/
/* this function creates a widget which edit or add a referential job */
GtkWidget* nc81_ref_job_edit_new(void)
{
	GtkWidget *container;
	GtkWidget *note_book;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkObject *adj;

	container = gtk_vbox_new(FALSE, 2);
	note_book = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(container), note_book, TRUE, TRUE, 1);

	/* page General */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("General"));

	/* job name field */
	field_name = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_name), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack(
			(nc81_job_name==NULL)? "New job": "Job",
			field_name, GTK_BOX(vbox));


	/* description field */
	field_description = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_description), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Description", field_description, GTK_BOX(vbox));

	/* path */
	field_path = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_path), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Path", field_path, GTK_BOX(vbox));

	/* argument */
	field_argument = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_argument), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Argument", field_argument, GTK_BOX(vbox));

	/* page Activity */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 3);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("Activity"));

	/* wad field */
	widget = gtk_check_button_new_with_label("Wait all dependencies");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_wad = widget;

	/* oec field */
	widget = gtk_check_button_new_with_label("On error continue");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_oec = widget;

	/* page tempo */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("Tempo"));
	/* start tempo */
	adj = gtk_adjustment_new(0, 0, 1000, 1, 2, 0);
	field_start_tempo = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_object_set_user_data(GTK_OBJECT(field_start_tempo), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Before execution", 
			field_start_tempo, GTK_BOX(vbox));

	/* end tempo */
	adj = gtk_adjustment_new(0, 0, 1000, 1, 2, 0);
	field_end_tempo = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_object_set_user_data(GTK_OBJECT(field_end_tempo), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("After execution", 
			field_end_tempo, GTK_BOX(vbox));

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_box_pack_end(GTK_BOX(container), hbox, FALSE, FALSE, 1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	nc81_ref_task_edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_name), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_description), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_wad), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_oec), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_path), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_argument), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_start_tempo), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_end_tempo), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	return container;
}
/****************************************************************************/
