/* file ref_job_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* ref_job_list;
static GtkWidget* field_job_name;
static GtkWidget* field_job_description;
static GtkWidget* field_job_path;
static GtkWidget* field_job_argument;

static GtkWidget* button_edit;
static GtkWidget* button_delete;
static GtkWidget* button_dep_list;
static GtkWidget* button_stat;

/****************************************************************************/
/* this callback function is called to field the field values */
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Job Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_name), fields[1]);

	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_description), fields[1]);

	if (strcmp(fields[0], "Path") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_path), fields[1]);

	if (strcmp(fields[0], "Argument") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_argument), fields[1]);

}


/****************************************************************************/
/* this function intializes the job fields
 * and copies the name of the selected job in the global variable
 */
void nc81_ref_job_display(gchar* job_name)
{
	gboolean enabled;

	nc81_job_select(job_name);

	if (nc81_ref_name == NULL || nc81_task_name == NULL 
			|| nc81_job_name == NULL)
	{
		/* no referential job selected : blank fields */
		gtk_entry_set_text(GTK_ENTRY(field_job_name), "");
		gtk_entry_set_text(GTK_ENTRY(field_job_description), "");
		gtk_entry_set_text(GTK_ENTRY(field_job_path), "");
		gtk_entry_set_text(GTK_ENTRY(field_job_argument), "");
		enabled = FALSE;
	}
	else
	{
		nc81_command_send("rji \"%r\" \"%t\" \"%j\"", 
				field_fill_callback,
				NULL,NULL,NULL,NULL);
		enabled = TRUE;
	}
	gtk_widget_set_sensitive(button_edit, enabled);
	gtk_widget_set_sensitive(button_delete, enabled);
	gtk_widget_set_sensitive(button_dep_list, enabled);
	gtk_widget_set_sensitive(button_stat, enabled);

}
/****************************************************************************/
/* this function is called when a job is selected */
void nc81_ref_job_list_select_child(GtkWidget* gtk_list, GtkWidget *list_item, 
		gpointer data)
{
	gchar* str;
	GtkWidget* label;
	label = g_list_nth_data(
			gtk_container_children(GTK_CONTAINER(list_item) ), 0);
	gtk_label_get(GTK_LABEL(label), &str);
	nc81_ref_job_display(str);
	/* TODO g_free(str)? */
}
/****************************************************************************/
/* this function is called when a job is unselected */
void nc81_ref_job_list_unselect_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	/* do not nc81_job_display(NULL)! */
}
/****************************************************************************/
/* this function is called for every line to add in the job list */
static void list_fill_callback(gchar** fields, void* data)
{
	gchar* str;
	GList *item_list;
	GtkWidget *item;
	str = g_strjoinv(" ", fields);
	/* create a list item */
	item = gtk_list_item_new_with_label(str);
	gtk_widget_show(GTK_WIDGET(item));
	/* add this item in the list */
	item_list = g_list_append(NULL, item); 
	gtk_list_append_items(GTK_LIST(ref_job_list), item_list);
	/* select the job */
	if (nc81_job_name != NULL && 
			strcmp(fields[0], nc81_job_name) == 0)
		gtk_list_select_child(GTK_LIST(ref_job_list), item);
	g_free(str);
}

/****************************************************************************/
/* this function updates the job list content, and the job fields */
void nc81_ref_job_view_update(void)
{
	gtk_list_clear_items(GTK_LIST(ref_job_list),0,-1);
	nc81_ref_job_display(nc81_job_name);
	nc81_command_send("rjl \"%r\" \"%t\"", 
			list_fill_callback, NULL, NULL, NULL, NULL);
}

/****************************************************************************/
static gint button_edit_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_job_edit);
	return 0;
}
/****************************************************************************/
static gint button_new_clicked(GtkWidget *widget, gpointer data)
{
	nc81_job_select(NULL);
	nc81_main_load(nc81_form_referential_job_edit);
	return 0;
}
/****************************************************************************/
static gint button_delete_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_job_delete);
	return 0;
}
/****************************************************************************/
static gint button_task_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_view);
	return 0;
}
/****************************************************************************/
static gint button_dep_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_job_dep_view);
	return 0;
}
/****************************************************************************/
static gint button_stat_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_sta_job_view);
	return 0;
}
/****************************************************************************/
static gint button_ref_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_view);
	return 0;
}
/****************************************************************************/
/* this function creates a widget which contains a list of all
 * referential jobs, and the job fields
 */
GtkWidget* nc81_ref_job_view_new(void)
{
	GtkWidget *container;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;

	container = gtk_vbox_new(FALSE,0);
	vbox1 = gtk_vbox_new(FALSE, 1);
	vbox2 = gtk_vbox_new(FALSE, 2);
	nc81_paned_new(vbox1, vbox2, container);

	/* referential name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);
	nc81_field_with_icon_new_pack("Referential", widget, 
			GTK_BOX(vbox1), nc81_icon_referential);


	/* task name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_task_name);
	nc81_field_with_icon_new_pack("Task", widget, 
			GTK_BOX(vbox1), nc81_icon_task);

	/* construct the job list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, FALSE, 1);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_job], 
			nc81_icon_bitmap_array[nc81_icon_job]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_label_new("Job list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_list_new();
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	ref_job_list = widget;

	gtk_signal_connect(GTK_OBJECT(ref_job_list),
			"select_child",
			nc81_ref_job_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(ref_job_list),
			"unselect_child",
			nc81_ref_job_list_unselect_child, NULL);

	/* construct job fields */

	/* job name */
	field_job_name = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_name), FALSE);
	nc81_field_new_pack("Job name", field_job_name, GTK_BOX(vbox2));

	/* description */
	field_job_description = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_description), FALSE);
	nc81_field_new_pack("Description", field_job_description, GTK_BOX(vbox2));

	/* path */
	field_job_path = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_path), FALSE);
	nc81_field_new_pack("Path", field_job_path, GTK_BOX(vbox2));

	/* argument */
	field_job_argument = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_argument), FALSE);
	nc81_field_new_pack("Argument", field_job_argument, GTK_BOX(vbox2));


	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Edit");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_edit_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_edit = menu_item;

	menu_item = gtk_menu_item_new_with_label("Add");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_new_clicked), NULL);

	menu_item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_delete_clicked), NULL);
	button_delete = menu_item;

	nc81_main_set_edit_menu(GTK_MENU(menu));

	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Referential");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_ref_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Task");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_task_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Job dependencies");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_dep_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_dep_list = menu_item;

	menu_item = gtk_menu_item_new_with_label("Job statistics");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_stat_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_stat = menu_item;


	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_ref_job_view_update();

	return container;
}
/****************************************************************************/
