/* file ref_task_oper_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"


static char* tb_operator[] = {
	"add",
	"add_from",
	"add_until",
	"remove",
	"remove_from",
	"remove_until",
	"shift_after",
	"shift_before",
	NULL
};

static char* tb_operation[] = {
	"day_of_week",
	"day_of_month",
	"day_of_year",
	"week_of_month",
	"week_of_year",
	"month_of_year",
	"day",
	"date",
	"always",
	"rule",
	"if_equal",
	"if_not_equal",
	NULL
};

static gboolean parent_is_task;
static GtkWidget *notebook;
static GtkWidget *field_operator;
static GtkWidget *field_rule;
static GtkWidget *field_dow; /* day_of_week */
static GtkWidget *field_dom; /* day_of_month */
static GtkWidget *field_doy; /* day_of_year */
static GtkWidget *field_wom; /* week_of_month */
static GtkWidget *field_woy; /* week_of_year */
static GtkWidget *field_moy; /* month_of_year */
static GtkWidget *field_day_d; /* day - day*/
static GtkWidget *field_day_m; /* day - month */
static GtkWidget *field_date_d; /* date - day */
static GtkWidget *field_date_m; /* date - month */
static GtkWidget *field_date_y; /* date - year */
static GtkWidget *field_ife_variable; /* if_equal - name */
static GtkWidget *field_ife_value; /* if_equal - value*/
static GtkWidget *field_ifn_variable; /* if_not_equal - name */
static GtkWidget *field_ifn_value; /* if_not_equal - value*/
static GList *combo_items;

/****************************************************************************/
static void valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar *arg;
	gchar *operator;
	gchar *name;
	gchar *value;
	int i,j,k;
	switch (gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)))
	{
		case 0: /* day_of_week */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_dow)));
			arg = g_strdup_printf("day_of_week \"%i\"", i);
			break;
		case 1: /* day_of_month */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_dom)));
			arg = g_strdup_printf("day_of_month \"%i\"", i);
			break;
		case 2: /* day_of_year */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_doy)));
			arg = g_strdup_printf("day_of_year \"%i\"", i);
			break;
		case 3: /* week_of_month */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_wom)));
			arg = g_strdup_printf("week_of_month \"%i\"", i);
			break;
		case 4: /* week_of_year */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_woy)));
			arg = g_strdup_printf("week_of_year \"%i\"", i);
			break;
		case 5: /* month_of_year */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_moy)));
			arg = g_strdup_printf("month_of_year \"%i\"", i);
			break;
		case 6: /* day */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_day_d)));
			j = atoi(gtk_entry_get_text(GTK_ENTRY(field_day_m)));
			arg = g_strdup_printf("day \"%02i-%02i\"",j,i);
			break;
		case 7: /* date */
			i = atoi(gtk_entry_get_text(GTK_ENTRY(field_date_d)));
			j = atoi(gtk_entry_get_text(GTK_ENTRY(field_date_m)));
			k = atoi(gtk_entry_get_text(GTK_ENTRY(field_date_y)));
			arg = g_strdup_printf("date \"%04i-%02i-%02i\"",k,j,i);
			break;
		case 8: /* always */
			arg = g_strdup("always");
			break;
		case 9: /* rule */
			name = gtk_entry_get_text(GTK_ENTRY(
						GTK_COMBO(field_rule)->entry));
			arg = g_strdup_printf("rule \"%s\"", name);
			break;
		case 10: /* if_equal */
			name = gtk_entry_get_text(GTK_ENTRY(
					GTK_COMBO(field_ife_variable)->entry));
			value = gtk_entry_get_text(GTK_ENTRY(field_ife_value));
			arg = g_strdup_printf("if_equal \"%s\" \"%s\"", 
					name, value);
			break;
		case 11: /* if_not_equal */
			name = gtk_entry_get_text(GTK_ENTRY(
					GTK_COMBO(field_ifn_variable)->entry));
			value = gtk_entry_get_text(GTK_ENTRY(field_ifn_value));
			arg = g_strdup_printf("if_not_equal \"%s\" \"%s\"", 
					name, value);
			break;
		
	}

	operator = gtk_entry_get_text(
			GTK_ENTRY(GTK_COMBO(field_operator)->entry));

	nc81_command_send((parent_is_task==TRUE)?
			"rtoa \"%r\" \"%t\" %x %y": 
			"rroa \"%r\" \"%R\" %x %y", 
		valid_callback, NULL, operator, arg, NULL);
	g_free(arg);

	nc81_main_load((parent_is_task == TRUE)?
			nc81_form_referential_task_operation:
			nc81_form_referential_rule_operation);

	return 0;
}
/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load((parent_is_task == TRUE)?
			nc81_form_referential_task_operation:
			nc81_form_referential_rule_operation);
	return 0;
}

/****************************************************************************/
static void nc81_operation_changed(GtkWidget *widget, gpointer data)
{
	int i;
	gchar* operation;
	operation = gtk_entry_get_text(GTK_ENTRY(widget));
	for (i = 0; tb_operation[i] != NULL; i++)
		if (strcmp(operation, tb_operation[i]) == 0)
		{
			gtk_notebook_set_page(GTK_NOTEBOOK(notebook), i);
			break;
		}
}
/****************************************************************************/
static void combo_init_callback(gchar** fields, void* data)
{
	gchar* str;
	str = g_strjoinv(" ", fields);
	combo_items = g_list_append(combo_items, str);
}
/****************************************************************************/
void init_fields(void)
{
	/* init rule list */
	combo_items = NULL;
	nc81_command_send("rrl \"%r\"", 
			combo_init_callback, 
			NULL, NULL, NULL,NULL);
	if (combo_items != NULL)
		gtk_combo_set_popdown_strings(GTK_COMBO(field_rule), 
				combo_items);

	/* init variable list */
	combo_items = NULL;
	nc81_command_send("vl", combo_init_callback, NULL, NULL, NULL,NULL);
	if (combo_items != NULL)
	{
		gtk_combo_set_popdown_strings(GTK_COMBO(field_ife_variable), 
			combo_items);
		gtk_combo_set_popdown_strings(GTK_COMBO(field_ifn_variable), 
			combo_items);
	}
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);
}
/****************************************************************************/
/* this function creates a widget to create a task operation */
static GtkWidget* nc81_ref_oper_edit_new(gboolean parent_task)
{
	GtkWidget *vbox;
	GtkWidget *page;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkObject *adj;
	GList *combo_items;
	int i;

	assert(nc81_ref_name != NULL);
	parent_is_task = parent_task;
	vbox = gtk_vbox_new(FALSE, 1);

	/* task/rule name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new((parent_is_task==TRUE)?"Task":"Rule");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE,1);
	gtk_entry_set_text(GTK_ENTRY(widget), 
			(parent_is_task==TRUE)?nc81_task_name:nc81_rule_name);

	/* referential name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new("Referential");
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);

	/* operator */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new("Operator");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	combo_items = NULL;
	for (i = 0; tb_operator[i] != NULL; i++)
		combo_items = g_list_append(combo_items, tb_operator[i]);
	if (combo_items != NULL)
		gtk_combo_set_popdown_strings(GTK_COMBO(widget), combo_items);
	gtk_combo_set_value_in_list(GTK_COMBO(widget), 1, 0); 
	field_operator = widget;

	/* operation */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new("Operation");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	combo_items = NULL;
	for (i = 0; tb_operation[i] != NULL; i++)
		combo_items = g_list_append(combo_items, tb_operation[i]);
	if (combo_items != NULL)
		gtk_combo_set_popdown_strings(GTK_COMBO(widget), combo_items);
	gtk_combo_set_value_in_list(GTK_COMBO(widget), 1, 0); 
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(widget)->entry), 
			"changed",
			GTK_SIGNAL_FUNC(nc81_operation_changed), NULL);

	/* notebook */
	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(vbox), notebook, FALSE, FALSE,1);
	gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	
	/* day of week  */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("Day of week");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(1, 1, 7, 1, 1, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	field_dow = widget;

	/* day of month */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("Day of month");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(1, -31, 31, 1, 3, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	field_dom = widget;

	/* day of year */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("Day of year");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(1, -366, 366, 1, 10, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	field_doy = widget;

	/* week of month*/
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("Week of month");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(1, -5, 5, 1, 1, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	field_wom = widget;

	/* week of year */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("Week of year");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(1, -52, 52, 1, 5, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	field_woy = widget;

	/* month of year */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("Month of year");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(1, -12, 12, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	field_moy = widget;

	/* day */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("day (dd/mm)");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);

	adj = gtk_adjustment_new(1, 1, 31, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	field_day_d = widget;

	adj = gtk_adjustment_new(1, 1, 12, 1, 1, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 5);
	field_day_m = widget;

	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));

	/* date */
	page = gtk_vbox_new(FALSE, 2);
	widget = gtk_label_new("date (dd/mm/yyyy)");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);

	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(page), hbox, FALSE, FALSE, 1);
	adj = gtk_adjustment_new(1, 1, 31, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	field_date_d = widget;

	adj = gtk_adjustment_new(1, 1, 12, 1, 1, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	field_date_m = widget;

	/* TODO select current year */
	adj = gtk_adjustment_new(2005, 2000, 3000, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 5);
	field_date_y = widget;

	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));

	/* always */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("No parameter to specify");
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));

	/* rule */
	page = gtk_hbox_new(FALSE, 2);
	widget = gtk_label_new("Rule");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_combo_set_value_in_list(GTK_COMBO(widget), 1, 0); 
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));
	field_rule = widget;

	/* if_equal */
	page = gtk_hbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));

	widget = gtk_label_new("Variable");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_combo_set_value_in_list(GTK_COMBO(widget), 1, 0); 
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	field_ife_variable = widget;

	widget = gtk_label_new("Value");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	field_ife_value = widget;

	/* if_not_equal */
	page = gtk_hbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			page, gtk_label_new(""));

	widget = gtk_label_new("Variable");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	widget = gtk_combo_new();
	gtk_combo_set_value_in_list(GTK_COMBO(widget), 1, 0); 
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	field_ifn_variable = widget;

	widget = gtk_label_new("Value");
	gtk_box_pack_start(GTK_BOX(page), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(page), widget, TRUE, TRUE, 1);
	field_ifn_value = widget;

	/* button box (ok/cancel) */
	hbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);

	/* button OK */
	widget = gtk_button_new_with_label("OK");
	gtk_container_add(GTK_CONTAINER(hbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);

	/* button Cancel */
	widget = gtk_button_new_with_label("Cancel");
	gtk_container_add(GTK_CONTAINER(hbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);

	init_fields();

	return vbox;
}
/****************************************************************************/
GtkWidget* nc81_ref_task_oper_edit_new(void)
{
	return nc81_ref_oper_edit_new(TRUE);
}
/****************************************************************************/
GtkWidget* nc81_ref_rule_oper_edit_new(void)
{
	return nc81_ref_oper_edit_new(FALSE);
}
/****************************************************************************/
