/* file ref_task_oper_simu.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static gboolean parent_is_task;
static guint8 tb[12][31];
static gchar* day_name_array[] = { "..","Su","Mo","Tu","We","Th","Fr","Sa"};
static gchar* month_name_array[] = { "Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"};

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load((parent_is_task == TRUE)?
			nc81_form_referential_task_operation:
			nc81_form_referential_rule_operation);
	return 0;
}
/****************************************************************************/
static void init_array_callback(gchar** fields, void* data)
{
	gint day,month, value;
	gchar* str;
	gchar* ptr;
	gchar day_name[3];
	str = g_strjoinv(" ", fields);

	/* ignore the two first lines */
	if (str[1] < '0' || str[1] > '9')
		return;
	
	/* get the day number */
	str[2] = '\0';
	day = atoi(str) - 1;
	assert(day >= 0 && day <= 31);
	
	/* example of a line : 27   Th    Su *  Su *  We    Fr    Mo*/

	/* read each column */
	ptr = str + 5;
	for (month = 0; month < 12 ; month++)
	{
		/* get day name */
		strncpy(day_name, ptr, 2);
		day_name[2] = '\0';
		for (value = 7; value > 0; value--)
			if (strcmp(day_name_array[value], day_name) == 0)
				break;
		/* is the day selected */
		if (ptr[3] == '*')
			value += 10;

		tb[month][day] = value;
		ptr += 6;
		
	}
	g_free(str);
}
/****************************************************************************/
void init_array(void)
{
	gchar* year;
	int i, j;
	for (i = 0; i < 12; i++)
		for (j = 0; j < 31; j++)
			tb[i][j] = 0;
	year = "2005";

	nc81_command_send(
			(parent_is_task)?
				"rtsy \"%r\" \"%t\" %x":
				"rrsy \"%r\" \"%R\" %x", 
			init_array_callback, 
			NULL, year, NULL,NULL);
}
/****************************************************************************/
static gint redraw_callback(GtkWidget *widget, GdkEventExpose *event)
{
	GdkFont *font;
	gint day, month;
	gboolean selected;
	gint day_index;
	gint value;
	gint x_border;
	gint y_border;
	gint cell_width;
	gint cell_height;
	gint x, y;
	gint x_size, y_size;
	GdkWindow *window;
	GdkColormap *colormap;
	GdkColor fg_color, bg_color;
	GdkColor wk_color, sl_color;
	GdkGC *gc;
	gboolean print_name;
	gint width, height;
	gchar* str;

	if (GTK_WIDGET_DRAWABLE(widget) == FALSE)
		return FALSE;

	window = widget->window;
	gc = gdk_gc_new(widget->window);
	gdk_gc_set_clip_rectangle(gc, &(event->area));

	font = gdk_font_load("-*-fixed-*-*-*-*-*-50-*-*-*-*-*-*");
	gdk_gc_set_font(gc, font);

	gdk_window_get_size(window, &width, &height);
	cell_width = (width - 1) / 13;
	cell_height = (height - 1) / 32;
	x_border = width - (cell_width * 12) - 1;
	y_border = height - (cell_height * 31) - 1;

	colormap = gdk_window_get_colormap(window);

	gdk_color_parse("white", &bg_color);
	gdk_colormap_alloc_color(colormap, &bg_color, FALSE, TRUE);

	gdk_color_parse("black", &fg_color);
	gdk_colormap_alloc_color(colormap, &fg_color, FALSE, TRUE);

	gdk_color_parse("PaleGreen", &wk_color);
	gdk_colormap_alloc_color(colormap, &wk_color, FALSE, TRUE);

	gdk_color_parse("DarkBlue", &sl_color);
	gdk_colormap_alloc_color(colormap, &sl_color, FALSE, TRUE);

	gdk_gc_set_background(gc, &bg_color);
	gdk_gc_set_foreground(gc, &fg_color);

	/* draw column names (if possible) : month */
	x_size = gdk_string_width(font, "XXX");
	y_size = gdk_string_height(font, "XXX");
	if (x_size < cell_width && y_size < cell_height)
	{
		y = y_border - (y_border - y_size) / 2;
		x = x_border;
		for (month = 0; month < 12; month++)
		{
			gdk_draw_string(window, font, gc, 
					x + (cell_width - x_size) / 2, 
					y,
					month_name_array[month]);
			x += cell_width;
		}
	}

	/* draw row names (if possible) : day 0-31 */
	x_size = gdk_string_width(font, "88");
	y_size = gdk_string_height(font, "88");
	if (x_size < cell_width && y_size < cell_height)
	{
		x = cell_width / 2;
		y = y_border + y_size;
		for (day = 0; day < 31; day++)
		{
			str = g_strdup_printf("%02i", day + 1);
			gdk_draw_string(window, font, gc, 
					x, y, str);
			g_free(str);
			y += cell_height;
		}
	}

	/* draw cells */
	print_name = (x_size < cell_width && y_size < cell_height)?TRUE:FALSE; 
	x = x_border;
	for (month = 0; month < 12; month++)
	{
		y = y_border;
		for (day = 0; day < 31; day++)
		{
			value = tb[month][day];
			if (value == 0)
				continue;
			selected = FALSE;
			if (value > 10) {
				selected = TRUE;
				day_index = value - 10;
			} else {
				selected = FALSE;
				day_index = value;
			} 
			if (selected == TRUE)
				gdk_gc_set_foreground(gc, &sl_color);
			else if (day_index == 1 || day_index == 7)
				gdk_gc_set_foreground(gc, &wk_color);
			else
				gdk_gc_set_foreground(gc, &bg_color);

			gdk_draw_rectangle(window, gc, 
					TRUE,
					x, y, 
					cell_width, 
					cell_height);

			/* print day name (if possible) */
			if (print_name)
			{
				gdk_gc_set_foreground(gc, 
						(selected)?&bg_color:&fg_color);
				gdk_draw_string(window, font, gc, 
					x + (cell_width - x_size) / 2, 
					y + cell_height - 
						(cell_height - y_size) / 2, 
					day_name_array[day_index]);
				gdk_gc_set_foreground(gc, &fg_color);
			}

			y += cell_height;
		}
		x += cell_width;
	}

	gdk_gc_destroy(gc);
	return 0;
}

/****************************************************************************/
/* this function creates a widget to simulate a task/rule operation */
static GtkWidget* nc81_ref_oper_simu_new(gboolean parent_task)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;

	assert(nc81_ref_name != NULL);
	parent_is_task = parent_task;
	vbox = gtk_vbox_new(FALSE, 1);

	/* task/rule name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new((parent_is_task==TRUE)?"Task":"Rule");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE,1);
	gtk_entry_set_text(GTK_ENTRY(widget), 
			(parent_is_task==TRUE)?nc81_task_name:nc81_rule_name);

	/* referential name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_label_new("Referential");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);


	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE,1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_drawing_area_new();
	gtk_widget_set_usize(widget, 250, 300);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);

	gtk_signal_connect(GTK_OBJECT(widget), "expose_event",
			GTK_SIGNAL_FUNC(redraw_callback), NULL);

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);

	/* button Close */
	widget = gtk_button_new_with_label("Close");
	gtk_container_add(GTK_CONTAINER(hbox), widget);
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);

	init_array();

	return vbox;
}
/****************************************************************************/
GtkWidget* nc81_ref_task_oper_simu_new(void)
{
	return nc81_ref_oper_simu_new(TRUE);
}
/****************************************************************************/
GtkWidget* nc81_ref_rule_oper_simu_new(void)
{
	return nc81_ref_oper_simu_new(FALSE);
}
/****************************************************************************/
