/* file ref_task_oper.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

gboolean parent_is_task;
GtkWidget *operation_list;
gint operation_index;

/****************************************************************************/
void operation_select(gint index)
{
	operation_index = index;
}
/****************************************************************************/
/* this function is called for every line to add in the operation list */
static void list_fill_callback(gchar** fields, void* data)
{
	gint row;
	gint i;
	gchar* values[5];
	/* initialize default value for each cell */
	for (i = 0; i < 5; i++)
		values[i] = "";
	/* copy fields values */
	for (i = 0; fields[i] != NULL; i++)
		values[i] = fields[i];
	row = gtk_clist_append(GTK_CLIST(operation_list), values);
}
/****************************************************************************/
/* this function is called when an operation is selected */
static void nc81_operation_list_select(GtkWidget* gtk_clist, 
		gint row, gint column, GdkEvent *event, gpointer data)
{
	operation_select(row);
}
/****************************************************************************/
/* this function is called when an operation is unselected */
static void nc81_operation_list_unselect(GtkWidget* gtk_clist, 
		gint row, gint column, GdkEvent *event, gpointer data)
{
	operation_select(-1);
	/* do not nc81_exp_display(NULL)! */
}
/****************************************************************************/
static void operation_list_init(void)
{
	operation_select(-1);
	gtk_clist_freeze(GTK_CLIST(operation_list));
	gtk_clist_clear(GTK_CLIST(operation_list));
	nc81_command_send(
			(parent_is_task) ? 
				"rtol \"%r\" \"%t\"":
				"rrol \"%r\" \"%R\"", 
			list_fill_callback, NULL, NULL, NULL,NULL);
	gtk_clist_thaw(GTK_CLIST(operation_list));
}
/****************************************************************************/
static gint button_add_clicked(GtkWidget *widget, gpointer data)
{
	if (parent_is_task == TRUE)
		nc81_main_load(nc81_form_referential_task_operation_edit);
	else
		nc81_main_load(nc81_form_referential_rule_operation_edit);
	return 0;
}
/****************************************************************************/
static gint button_simu_clicked(GtkWidget *widget, gpointer data)
{
	if (parent_is_task == TRUE)
		nc81_main_load(nc81_form_referential_task_operation_simu);
	else
		nc81_main_load(nc81_form_referential_rule_operation_simu);
	return 0;
}
/****************************************************************************/
static gint button_delete_clicked(GtkWidget *widget, gpointer data)
{
	gchar* str;
	if (operation_index == -1)
		return 0;
	str = g_strdup_printf("%i", operation_index + 1);
	nc81_command_send((parent_is_task==TRUE) ? 
				"rtok \"%r\" \"%t\" %x" :
				"rrok \"%r\" \"%R\" %x",
			list_fill_callback, NULL, str, NULL, NULL);
	g_free(str);
	operation_list_init();
	return 0;
}
/****************************************************************************/
static gint button_update_clicked(GtkWidget *widget, gpointer data)
{
	operation_list_init();
	return 0;
}
/****************************************************************************/
static gint button_up_clicked(GtkWidget *widget, gpointer data)
{
	gchar* str;
	if (operation_index == -1)
		return 0;
	str = g_strdup_printf("%i", operation_index + 1);
	nc81_command_send((parent_is_task==TRUE)?
			"rtom \"%r\" \"%t\" %x up":
			"rrom \"%r\" \"%R\" %x up",
			list_fill_callback, NULL, str, NULL, NULL);
	g_free(str);
	operation_list_init();
	return 0;
}
/****************************************************************************/
static gint button_down_clicked(GtkWidget *widget, gpointer data)
{
	gchar* str;
	if (operation_index == -1)
		return 0;
	str = g_strdup_printf("%i", operation_index + 1);
	nc81_command_send((parent_is_task==TRUE)?
			"rtom \"%r\" \"%t\" %x down":
			"rrom \"%r\" \"%R\" %x down",
			list_fill_callback, NULL, str, NULL, NULL);
	g_free(str);
	operation_list_init();
	return 0;
}
/****************************************************************************/
static gint button_ref_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_view);
	return 0;
}
/****************************************************************************/
static gint button_parent_clicked(GtkWidget *widget, gpointer data)
{
	if (parent_is_task == TRUE)
		nc81_main_load(nc81_form_referential_task_view);
	else
		nc81_main_load(nc81_form_referential_rule_view);
	return 0;
}

/****************************************************************************/
/* this function creates a widget to edit task operations */
static GtkWidget* nc81_ref_oper_new(gboolean parent_task)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;
	gchar* titles[4];

	assert(nc81_ref_name != NULL);
	parent_is_task = parent_task;

	vbox = gtk_vbox_new(FALSE, 1);

	/* referential name */
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_referential], 
			nc81_icon_bitmap_array[nc81_icon_referential]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_label_new("Referential");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 1);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);

	/* task/rule name */
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
	/* TODO change icon task/rule */
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_label_new((parent_is_task==TRUE)?"Task":"Rule");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 1);
	gtk_entry_set_text(GTK_ENTRY(widget), 
			(parent_is_task)?nc81_task_name:nc81_rule_name);

	/* operation list */
	widget = gtk_label_new("Operation list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 1);
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	titles[0] = "Operator";
	titles[1] = "Operation";
	titles[2] = "Argument 1";
	titles[3] = "Argument 2";
	widget = gtk_clist_new_with_titles(4,titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 70);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 90);
	gtk_clist_set_column_width(GTK_CLIST(widget), 2, 70);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled), widget);
	operation_list = widget;

	gtk_signal_connect(GTK_OBJECT(operation_list), "select_row",
			nc81_operation_list_select, NULL);
	gtk_signal_connect(GTK_OBJECT(operation_list), "unselect_row",
			nc81_operation_list_unselect, NULL);
	
	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Up");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_up_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Down");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_down_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Add");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_add_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);


	menu_item = gtk_menu_item_new_with_label("Delete");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_delete_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);


	menu_item = gtk_menu_item_new_with_label("Update");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_update_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Simulation");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_simu_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	nc81_main_set_edit_menu(GTK_MENU(menu));



	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Referential");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_ref_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label(
			(parent_is_task==TRUE)?"Task":"Rule");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_parent_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	nc81_main_set_select_menu(GTK_MENU(menu));

	/* init operation list */
	operation_list_init();

	return vbox;
}
/****************************************************************************/
GtkWidget* nc81_ref_task_oper_new(void)
{
	return nc81_ref_oper_new(TRUE);
}
/****************************************************************************/
GtkWidget* nc81_ref_rule_oper_new(void)
{
	return nc81_ref_oper_new(FALSE);
}
/****************************************************************************/
