/* file ref_rule_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* ref_rule_list;

static GtkWidget* button_edit;
static GtkWidget* button_delete;
static GtkWidget* button_operation_list;
/****************************************************************************/
void nc81_ref_rule_display(gchar* rule_name)
{
	gboolean enabled;
	nc81_rule_select(rule_name);
	if (nc81_ref_name == NULL || nc81_rule_name == NULL)
		enabled = FALSE;
	else
		enabled = TRUE;
	gtk_widget_set_sensitive(button_edit, enabled);
	gtk_widget_set_sensitive(button_delete, enabled);
	gtk_widget_set_sensitive(button_operation_list, enabled);
}
/****************************************************************************/
/* this function is called when a rule is selected */
void nc81_ref_rule_list_select_child(GtkWidget* gtk_list, GtkWidget *list_item, 
		gpointer data)
{
	gchar* str;
	GtkWidget* label;
	label = g_list_nth_data(
			gtk_container_children(GTK_CONTAINER(list_item) ), 0);
	gtk_label_get(GTK_LABEL(label), &str);
	nc81_ref_rule_display(str);
	/* TODO g_free(str)? */
}
/****************************************************************************/
/* this function is called when a rule is unselected */
void nc81_ref_rule_list_unselect_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	/* do not nc81_ref_display(NULL)! */
}
/****************************************************************************/
/* this function is called for every line to add in the rule list */
static void list_fill_callback(gchar** fields, void* data)
{
	gchar* str;
	GList *item_list;
	GtkWidget *item;
	str = g_strjoinv(" ", fields);
	/* create a list item */
	item = gtk_list_item_new_with_label(str);
	gtk_widget_show(GTK_WIDGET(item));
	/* add this item in the list */
	item_list = g_list_append(NULL, item); 
	gtk_list_append_items(GTK_LIST(ref_rule_list), item_list);
	/* select the rule*/
	if (nc81_rule_name != NULL && 
			strcmp(fields[0], nc81_rule_name) == 0)
		gtk_list_select_child(GTK_LIST(ref_rule_list), item);
	g_free(str);
}

/****************************************************************************/
/* this function updates the rule list content*/
void nc81_ref_rule_view_update(void)
{
	gtk_list_clear_items(GTK_LIST(ref_rule_list),0,-1);
	nc81_ref_rule_display(nc81_rule_name);
	nc81_command_send("rrl \"%r\"", list_fill_callback, 
			NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static gint button_edit_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_rule_edit);
	return 0;
}
/****************************************************************************/
static gint button_new_clicked(GtkWidget *widget, gpointer data)
{
	nc81_rule_select(NULL);
	nc81_main_load(nc81_form_referential_rule_edit);
	return 0;
}
/****************************************************************************/
static gint button_delete_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_rule_delete);
	return 0;
}
/****************************************************************************/
static gint button_ref_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_view);
	return 0;
}
/****************************************************************************/
static gint button_operation_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_rule_operation);
	return 0;
}

/****************************************************************************/
/* this function creates a widget which contains a list of all
 * referential rules.
 */
GtkWidget* nc81_ref_rule_view_new(void)
{
	GtkWidget *container;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;

	container = gtk_vbox_new(FALSE,0);

	/* referential name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);
	nc81_field_with_icon_new_pack("Referential", widget, 
			GTK_BOX(container), nc81_icon_referential);

	/* construct the rule list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(container), 1);
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, TRUE, 1);

	/* TODO change icon */
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_label_new("Rule list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(container), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_list_new();
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	ref_rule_list = widget;

	gtk_signal_connect(GTK_OBJECT(ref_rule_list),
			"select_child",
			nc81_ref_rule_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(ref_rule_list),
			"unselect_child",
			nc81_ref_rule_list_unselect_child, NULL);

	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Edit");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_edit_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_edit = menu_item;

	menu_item = gtk_menu_item_new_with_label("Add");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_new_clicked), NULL);

	menu_item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_delete_clicked), NULL);
	button_delete = menu_item;

	nc81_main_set_edit_menu(GTK_MENU(menu));


	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Referential");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_ref_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Operations");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_operation_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_operation_list = menu_item;

	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_ref_rule_view_update();
	return container;
}
/****************************************************************************/
