/* file ref_task_copy.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

GtkWidget *field_new_name;
GtkWidget *field_referential;
static GList *combo_items;

/****************************************************************************/
static void combo_init_callback(gchar** fields, void* data)
{
	gchar* str;
	str = g_strjoinv(" ", fields);
	combo_items = g_list_append(combo_items, str);
}
/****************************************************************************/
static void referential_combo_init()
{
	combo_items = NULL;
	nc81_command_send("rl", combo_init_callback, NULL, NULL, NULL,NULL);
	gtk_combo_set_popdown_strings(GTK_COMBO(field_referential), 
			combo_items);
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);
	/* select the current referential */
	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(field_referential)->entry),
				nc81_ref_name);
}
/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* new_name;
	gchar* ref_name;
	new_name = gtk_entry_get_text(GTK_ENTRY(field_new_name));
	ref_name = gtk_entry_get_text(GTK_ENTRY(
				GTK_COMBO(field_referential)->entry));
	/* copy the referential task */
	nc81_command_send("rtd \"%r\" \"%t\" \"%x\" \"%y\"", 
		field_name_valid_callback,
		NULL, ref_name, new_name, NULL);
	nc81_task_select(new_name);

	/* select the new task with its referential */
	nc81_ref_select(ref_name);
	nc81_task_select(new_name);

	/* TODO do not close if error */
	nc81_main_load(nc81_form_referential_task_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_view);
	return 0;
}

/****************************************************************************/
/* this function creates a widget which copy a rerential task */
GtkWidget* nc81_ref_task_copy_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	gchar* str;
	vbox = gtk_vbox_new(FALSE, 2);

	assert(nc81_ref_name != NULL);
	/* the question */
	str = g_strdup_printf("The task '%s' of the referential '%s' "\
			"will be copied. Enter the name of the new task, "\
			"and select its referential.",
			nc81_task_name, nc81_ref_name);
	widget = gtk_label_new(str);
	g_free(str);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 10);
	gtk_label_set_line_wrap(GTK_LABEL(widget), TRUE);

	/* the task name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 15);
	widget = gtk_label_new("New task");
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 5);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	gtk_widget_grab_focus(widget);
	field_new_name = widget;

	/* referential list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);
	widget = gtk_label_new("Referential");
	gtk_label_set_justify(GTK_LABEL(widget), GTK_JUSTIFY_RIGHT);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	widget = gtk_combo_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	gtk_object_set_user_data(GTK_OBJECT(widget), GUINT_TO_POINTER(FALSE));
	field_referential = widget;

	/* button box */
	widget = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 10);
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	referential_combo_init();

	return vbox;
}
/****************************************************************************/
