/* file ref_task_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget *field_name;
static GtkWidget *field_description;
static GtkWidget *field_target;
static GtkWidget *field_queue;
static GtkWidget *field_enabled;
static GtkWidget *field_wad; /* wait all dependencies */
static GtkWidget *field_oec; /* on error continue */
static GtkWidget *field_iteration;
static GtkWidget *field_interval;
static GtkWidget *field_from;
static GtkWidget *field_lock_variable;
static GtkWidget *field_lock_action;
static GList *combo_items;

/****************************************************************************/
static void combo_init_callback(gchar** fields, void* data)
{
	gchar* str;
	str = g_strjoinv(" ", fields);
	combo_items = g_list_append(combo_items, str);
}
/****************************************************************************/
static void variable_combo_init()
{
	gchar* str;
	combo_items = NULL;
	/* add `none' at the begining of the list */
	str = g_strdup("none");
	combo_items = g_list_append(combo_items, str);
	nc81_command_send("vl", combo_init_callback, NULL, NULL, NULL,NULL);
	gtk_combo_set_popdown_strings(GTK_COMBO(field_lock_variable), 
			combo_items);
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);
}
/****************************************************************************/
static void target_combo_init()
{
	combo_items = NULL;
	nc81_command_send("tl", combo_init_callback, NULL, NULL, NULL,NULL);
	gtk_combo_set_popdown_strings(GTK_COMBO(field_target), combo_items);
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);
}
/****************************************************************************/
static void queue_combo_init()
{
	gchar* target_name;
	target_name = gtk_entry_get_text(
			GTK_ENTRY(GTK_COMBO(field_target)->entry));	
	combo_items = NULL;
	nc81_command_send("ql \"%x\"", 
			combo_init_callback, NULL, target_name, NULL,NULL);
	gtk_combo_set_popdown_strings(GTK_COMBO(field_queue), combo_items);
	/* TODO is it necessary to free each item ?*/
	g_list_free(combo_items);
}
/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */
	if (strcmp(fields[0], "Task Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);

	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_description), fields[1]);
	
	if (strcmp(fields[0], "Target") == 0)
	{
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(field_target)->entry), 
				fields[1]);
		queue_combo_init();	
	}
	if (strcmp(fields[0], "Queue") == 0)
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(field_queue)->entry), 
				fields[1]);

	if (strcmp(fields[0], "Enabled") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_enabled), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "Wait All Dependencies") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_wad), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "On Error Continue") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_oec), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

	if (strcmp(fields[0], "Iterations") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_iteration), 
				fields[1]);
	if (strcmp(fields[0], "Interval") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_interval), 
				fields[1]);
	if (strcmp(fields[0], "From") == 0)
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(field_from)->entry), 
				fields[1]);
	if (strcmp(fields[0], "Lock Action") == 0)
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(
				field_lock_action)->entry), fields[1]);
	if (strcmp(fields[0], "Lock Variable") == 0)
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(
				field_lock_variable)->entry), fields[1]);
}
/****************************************************************************/
/* this function updates the task fields */
static void nc81_ref_task_edit_update(void)
{
	variable_combo_init();
	target_combo_init();
	queue_combo_init();
	if (nc81_task_name == NULL) {
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(field_enabled),
				TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(field_wad),
				TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(field_oec),
				TRUE);
		return;
	}
	nc81_command_send("rti \"%r\" \"%t\"", 
			field_fill_callback, NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* task_name;
	gchar* value;
	gboolean changed;
	gboolean bool_value;
	gboolean target_changed;
	gboolean queue_changed;
	gboolean interval_changed;
	gboolean iteration_changed;
	gboolean from_changed;
	gboolean lock_variable_changed;
	gboolean lock_action_changed;
	gchar* iteration;
	gchar* interval;
	gchar* from;
	gchar* target_name;
	gchar* queue_name;
	gchar* lock_variable;
	gchar* lock_action;

	/* referential name */
	task_name = gtk_entry_get_text(GTK_ENTRY(field_name));
	if (nc81_task_name == NULL)
	{
		/* create a new task */
		nc81_command_send( "rta \"%r\" \"%x\"",  
			field_name_valid_callback,
			NULL, task_name, NULL,NULL);
		nc81_task_select(task_name);
	}
	else
	{
		changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
					GTK_OBJECT(field_name)));
		if (changed == TRUE)
		{
			/* change the name of the task */
			nc81_command_send("rtsn \"%r\" \"%t\" \"%x\"", 
				field_name_valid_callback,
				NULL, task_name, NULL,NULL);
			nc81_task_select(task_name);
		}
	}

	/* description field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_description)));
	if (changed == TRUE)
	{
		value = gtk_entry_get_text(GTK_ENTRY(field_description));
		nc81_command_send("rtsd \"%r\" \"%t\" \"%x\"", 
			field_name_valid_callback, NULL, 
			value,NULL,NULL);
	}

	/* target and queue */
	target_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(GTK_COMBO(field_target)->entry)));
	queue_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(GTK_COMBO(field_queue)->entry)));
	if (target_changed == TRUE || queue_changed == TRUE)
	{
		target_name = gtk_entry_get_text(
			GTK_ENTRY(GTK_COMBO(field_target)->entry));
		queue_name = gtk_entry_get_text(
			GTK_ENTRY(GTK_COMBO(field_queue)->entry));
		nc81_command_send("rtst \"%r\" \"%t\" \"%x\" \"%y\"", 
			field_name_valid_callback, NULL, 
			target_name, queue_name,NULL);
	}

	/* enabled field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_enabled)));
	if (changed == TRUE)
	{
		bool_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_enabled));
		nc81_command_send("rtse \"%r\" \"%t\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(bool_value == TRUE)?"true":"false",
			NULL,NULL);
	}

	/* wad field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_wad)));
	if (changed == TRUE)
	{
		bool_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_wad));
		nc81_command_send("rtswad \"%r\" \"%t\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(bool_value == TRUE)?"true":"false",
			NULL,NULL);
	}

	/* oec field */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_oec)));
	if (changed == TRUE)
	{
		bool_value = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_oec));
		nc81_command_send("rtsoec \"%r\" \"%t\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(bool_value == TRUE)?"true":"false",
			NULL,NULL);
	}

	/* iteration, interval and from fields */
	iteration_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_iteration)));
	interval_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_interval)));
	from_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(GTK_COMBO(field_from)->entry)));
	if (interval_changed || iteration_changed || from_changed)
	{
		iteration = gtk_entry_get_text(GTK_ENTRY(field_iteration));
		interval = gtk_entry_get_text(GTK_ENTRY(field_interval));
		from = gtk_entry_get_text(GTK_ENTRY(
					GTK_COMBO(field_from)->entry));
		nc81_command_send("rtsc \"%r\" \"%t\" \"%x\" \"%y\" \"%z\"", 
			field_name_valid_callback, NULL, 
			iteration, interval,from);
	}
	/* lock variable / action fields */
	lock_action_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
			GTK_OBJECT(GTK_COMBO(field_lock_action)->entry)));
	lock_variable_changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
			GTK_OBJECT(GTK_COMBO(field_lock_variable)->entry)));
	if (lock_action_changed || lock_variable_changed)
	{
		lock_action = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(
					field_lock_action)->entry));
		lock_variable = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(
					field_lock_variable)->entry));
		if (strcmp(lock_action, "none") == 0)
			nc81_command_send("rtsl \"%r\" \"%t\" none", 
				field_name_valid_callback, NULL, 
				NULL, NULL, NULL);
		else
			nc81_command_send("rtsl \"%r\" \"%t\" \"%x\" \"%y\"", 
				field_name_valid_callback, NULL, 
				lock_variable, lock_action, NULL);
	}

	/* TODO do not close if error */
	nc81_main_load(nc81_form_referential_task_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_view);
	return 0;
}
/****************************************************************************/
static void nc81_field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}
/****************************************************************************/
static void nc81_field_target_lost_focus_callback(GtkWidget *field, gpointer data)
{
	gchar* target_name;
	gboolean changed;
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(GTK_COMBO(field_target)->entry)));
	if (changed == FALSE)
		return;
	/* change the selected target and init the queue list */
	target_name = gtk_entry_get_text(
			GTK_ENTRY(GTK_COMBO(field_target)->entry));
	nc81_target_select(target_name);
	queue_combo_init();
}

/****************************************************************************/
/* this function creates a widget which edit or add a referential task */
GtkWidget* nc81_ref_task_edit_new(void)
{
	GtkWidget *note_book;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkObject *adj;
	GtkWidget *container;
	GList *items;

	container = gtk_vbox_new(FALSE, 2);
	note_book = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(container), note_book, TRUE, TRUE, 1);

	/* page General */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("General"));

	/* task name field */
	field_name = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_name), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack(
			(nc81_task_name==NULL)? "New task": "Task",
			field_name, GTK_BOX(vbox));

	/* description field */
	field_description = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_description), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Description", field_description, GTK_BOX(vbox));


	/* target field */
	field_target = gtk_combo_new();
	gtk_object_set_user_data(GTK_OBJECT(field_target), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Target", field_target, GTK_BOX(vbox));

	/* queue field */
	field_queue = gtk_combo_new();
	gtk_object_set_user_data(GTK_OBJECT(field_queue), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Queue", field_queue, GTK_BOX(vbox));

	/* page activity */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("Activity"));
	/* enabled field */
	widget = gtk_check_button_new_with_label("Enabled");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_enabled= widget;

	/* wad field */
	widget = gtk_check_button_new_with_label("Wait all dependencies");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_wad = widget;

	/* oec field */
	widget = gtk_check_button_new_with_label("On error continue");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_oec = widget;

	/* page Iterations*/
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("Iteration"));

	/* number of iterations*/
	adj = gtk_adjustment_new(1, 0, 1000, 1, 2, 0);
	field_iteration = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(field_iteration), TRUE);
	gtk_object_set_user_data(GTK_OBJECT(field_iteration), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Number", field_iteration, GTK_BOX(vbox));

	/* interval */
	adj = gtk_adjustment_new(1, 0, 1000, 1, 2, 0);
	field_interval = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(field_interval), TRUE);
	gtk_object_set_user_data(GTK_OBJECT(field_interval), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Interval", field_interval, GTK_BOX(vbox));

	/* from */
	field_from = gtk_combo_new();
	gtk_object_set_user_data(GTK_OBJECT(field_from), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("From", field_from, GTK_BOX(vbox));
	items = NULL;
	items = g_list_append(items, "begin");
	items = g_list_append(items, "end");
	gtk_combo_set_popdown_strings(GTK_COMBO(field_from), items);

	/* page Lock*/
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
	gtk_notebook_append_page(GTK_NOTEBOOK(note_book), vbox,
			gtk_label_new("Lock"));

	/* lock action */
	field_lock_action = gtk_combo_new();
	gtk_object_set_user_data(GTK_OBJECT(field_lock_action), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Action", field_lock_action, GTK_BOX(vbox));
	items = NULL;
	items = g_list_append(items, "none");
	items = g_list_append(items, "lock");
	items = g_list_append(items, "unlock");
	gtk_combo_set_popdown_strings(GTK_COMBO(field_lock_action), items);

	/* lock variable */
	field_lock_variable = gtk_combo_new();
	gtk_object_set_user_data(GTK_OBJECT(field_lock_variable), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Variable", field_lock_variable, GTK_BOX(vbox));

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_box_pack_end(GTK_BOX(container), hbox, FALSE, TRUE, 1);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	nc81_ref_task_edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_name), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_description), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(field_target)->entry), 
			"changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(field_queue)->entry), 
			"changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_iteration), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_interval), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(field_from)->entry), 
			"changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_enabled), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_wad), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_oec), "clicked",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(field_lock_action)->entry), 
			"changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(field_lock_variable)->entry), 
			"changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);

	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(field_target)->entry), 
			"focus_out_event",
			GTK_SIGNAL_FUNC(nc81_field_target_lost_focus_callback), NULL);
	return container;
}
/****************************************************************************/
