/* file ref_task_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* ref_task_list;
static GtkWidget* field_task_name;
static GtkWidget* field_task_description;
static GtkWidget* field_task_target;
static GtkWidget* field_task_queue;

static GtkWidget* button_edit;
static GtkWidget* button_copy;
static GtkWidget* button_delete;
static GtkWidget* button_stat;
static GtkWidget* button_job_list;
static GtkWidget* button_condition_list;
static GtkWidget* button_operation_list;
static GtkWidget* button_dependencies_list;
static GtkWidget* button_graph;

/****************************************************************************/
/* this callback function is called to field the field values */
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Task Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_task_name), fields[1]);

	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_task_description), fields[1]);

	if (strcmp(fields[0], "Target") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_task_target), fields[1]);

	if (strcmp(fields[0], "Queue") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_task_queue), fields[1]);

}


/****************************************************************************/
/* this function intializes the task fields
 * and copies the name of the selected task in the global variable
 */
void nc81_ref_task_display(gchar* task_name)
{
	gboolean enabled;

	nc81_task_select(task_name);

	if (nc81_ref_name == NULL || nc81_task_name == NULL)
	{
		/* no referential task selected : blank fields */
		gtk_entry_set_text(GTK_ENTRY(field_task_name), "");
		gtk_entry_set_text(GTK_ENTRY(field_task_description), "");
		gtk_entry_set_text(GTK_ENTRY(field_task_target), "");
		gtk_entry_set_text(GTK_ENTRY(field_task_queue), "");
		enabled = FALSE;
	}
	else
	{
		nc81_command_send("rti \"%r\" \"%t\"", field_fill_callback,
				NULL,NULL,NULL, NULL);
		enabled = TRUE;
	}
	gtk_widget_set_sensitive(button_edit, enabled);
	gtk_widget_set_sensitive(button_delete, enabled);
	gtk_widget_set_sensitive(button_copy, enabled);
	gtk_widget_set_sensitive(button_stat, enabled);
	gtk_widget_set_sensitive(button_job_list, enabled);
	gtk_widget_set_sensitive(button_condition_list, enabled);
	gtk_widget_set_sensitive(button_operation_list, enabled);
	gtk_widget_set_sensitive(button_dependencies_list, enabled);
	gtk_widget_set_sensitive(button_graph, enabled);

}
/****************************************************************************/
/* this function is called when a task is selected */
void nc81_ref_task_list_select_child(GtkWidget* gtk_clist, gint row, 
		gint column, GdkEvent *event, gpointer data)
{
	gchar* str;
	/* row 0 is the task name */
	gtk_clist_get_text(GTK_CLIST(ref_task_list), row, 0 ,&str);
	nc81_ref_task_display(str);
}
/****************************************************************************/
/* this function is called when a task is unselected */
/****************************************************************************/
void nc81_ref_task_list_unselect_child(GtkWidget* gtk_clist, gint row, 
		gint column, GdkEvent *event, gpointer data)
{
	/* do not nc81_ref_display(NULL)! */
}
/* this function is called for every line to add in the task list */
static void list_fill_callback(gchar** fields, void* data)
{
	gint row;
	gchar* values[3];
	values[0] = fields[0]; /* name */
	values[1] = fields[1]; /* description */
	row = gtk_clist_append(GTK_CLIST(ref_task_list), values);
	/* select the task*/
	if (nc81_task_name != NULL && 
			strcmp(fields[0], nc81_task_name) == 0)
		gtk_clist_select_row(GTK_CLIST(ref_task_list), row, 0);

}

/****************************************************************************/
/* this function updates the task list content, and the task fields */
void nc81_ref_task_view_update(void)
{
	gtk_clist_freeze(GTK_CLIST(ref_task_list));
	gtk_clist_clear(GTK_CLIST(ref_task_list));
	nc81_ref_task_display(nc81_task_name);
	nc81_command_send("rtl -d \"%r\"", list_fill_callback, 
			NULL, NULL, NULL,NULL);
	gtk_clist_thaw(GTK_CLIST(ref_task_list));

}

/****************************************************************************/
static gint button_edit_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_edit);
	return 0;
}
/****************************************************************************/
static gint button_new_clicked(GtkWidget *widget, gpointer data)
{
	nc81_task_select(NULL);
	nc81_main_load(nc81_form_referential_task_edit);
	return 0;
}
/****************************************************************************/
static gint button_delete_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_delete);
	return 0;
}
/****************************************************************************/
static gint button_copy_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_copy);
	return 0;
}
/****************************************************************************/
static gint button_ref_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_view);
	return 0;
}
/****************************************************************************/
static gint button_job_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_job_view);
	return 0;
}
/****************************************************************************/
static gint button_condition_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_condition_view);
	return 0;
}
/****************************************************************************/
static gint button_dependencies_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_dep_view);
	return 0;
}
/****************************************************************************/
static gint button_stat_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_sta_task_view);
	return 0;
}
/****************************************************************************/
static gint button_graph_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_graph_ref_job);
	return 0;
}
/****************************************************************************/
static gint button_operation_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_operation);
	return 0;
}


/****************************************************************************/
/* this function creates a widget which contains a list of all
 * referential tasks, and the task fields
 */
GtkWidget* nc81_ref_task_view_new(void)
{
	GtkWidget *container;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;
	gchar* titles[3];

	container = gtk_vbox_new(FALSE,0);
	vbox1 = gtk_vbox_new(FALSE, 1);
	vbox2 = gtk_vbox_new(FALSE, 2);
	nc81_paned_new(vbox1, vbox2, container);

	/* referential name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);
	nc81_field_with_icon_new_pack("Referential", widget, 
			GTK_BOX(vbox1), nc81_icon_referential);

	/* construct the task list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 1);

	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_label_new("Task list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	titles[0] = "Name";
	titles[1] = "Description";
	widget = gtk_clist_new_with_titles(2,titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 90);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 200);
	gtk_container_add(GTK_CONTAINER(scrolled), widget);
	ref_task_list = widget;

	gtk_signal_connect(GTK_OBJECT(ref_task_list), "select_row",
			nc81_ref_task_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(ref_task_list), "unselect_row",
			nc81_ref_task_list_unselect_child, NULL);

	field_task_name = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_task_name), FALSE);
	nc81_field_new_pack("Task name", field_task_name, GTK_BOX(vbox2));

	field_task_description = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_task_description), FALSE);
	nc81_field_new_pack("Description", field_task_description, GTK_BOX(vbox2));

	field_task_target = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_task_target), FALSE);
	nc81_field_new_pack("Target", field_task_target, GTK_BOX(vbox2));

	field_task_queue = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_task_queue), FALSE);
	nc81_field_new_pack("Queue", field_task_queue, GTK_BOX(vbox2));

	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Edit");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_edit_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_edit = menu_item;

	menu_item = gtk_menu_item_new_with_label("Add");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_new_clicked), NULL);

	menu_item = gtk_menu_item_new_with_label("Copy");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_copy_clicked), NULL);
	button_copy = menu_item;

	menu_item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_delete_clicked), NULL);
	button_delete = menu_item;

	nc81_main_set_edit_menu(GTK_MENU(menu));


	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Referential");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_ref_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);
	
	menu_item = gtk_menu_item_new_with_label("Jobs");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_job_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_job_list = menu_item;
	
	menu_item = gtk_menu_item_new_with_label("Operations");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_operation_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_operation_list = menu_item;
	
	menu_item = gtk_menu_item_new_with_label("Conditions");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_condition_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_condition_list = menu_item;

	menu_item = gtk_menu_item_new_with_label("Task dependencies");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_dependencies_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_stat = menu_item;

	menu_item = gtk_menu_item_new_with_label("Task statistics");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_stat_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_dependencies_list = menu_item;

	menu_item = gtk_menu_item_new_with_label("Graph");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_graph_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_graph = menu_item;



	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_ref_task_view_update();
	return container;
}
/****************************************************************************/
