/* file sta_task_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* sta_task_list;
static GtkWidget* field_mini_duration;
static GtkWidget* field_maxi_duration;
static GtkWidget* field_average_duration;
static GtkWidget* duration_list;

static GtkWidget* button_job_list;
static GtkWidget* button_referential;

/****************************************************************************/
/* this function is called for every line to add in the duration list */
static void duration_list_fill_callback(gchar** fields, void* data)
{
	gint row;
	gchar* values[2];
	values[1] = fields[0]; /* duration */
	values[0] = fields[1]; /* date */
	row = gtk_clist_append(GTK_CLIST(duration_list), values);
}

/****************************************************************************/
/* this callback function is called to field the field values */
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "average") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_average_duration), fields[1]);
	if (strcmp(fields[0], "mini") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_mini_duration), fields[1]);
	if (strcmp(fields[0], "maxi") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_maxi_duration), fields[1]);

}


/****************************************************************************/
/* this function intializes the task fields
 * and copies the name of the selected task in the global variable
 */
static void nc81_sta_task_display(gchar* task_name)
{
	gboolean enabled;

	nc81_task_select(task_name);

	if (nc81_ref_name == NULL || nc81_task_name == NULL)
	{
		/* no statistics task selected : blank fields */
		gtk_entry_set_text(GTK_ENTRY(field_average_duration), "");
		gtk_entry_set_text(GTK_ENTRY(field_mini_duration), "");
		gtk_entry_set_text(GTK_ENTRY(field_maxi_duration), "");
		enabled = FALSE;
	}
	else
	{
		nc81_command_send("sti -h \"%r\" \"%t\"", field_fill_callback,
				NULL,NULL,NULL, NULL);
		/* initialize the duration list */
		gtk_clist_freeze(GTK_CLIST(duration_list));
		gtk_clist_clear(GTK_CLIST(duration_list));
		nc81_command_send("sti -hd \"%r\" \"%t\"", 
				duration_list_fill_callback, 
				NULL, NULL, NULL,NULL);
		gtk_clist_thaw(GTK_CLIST(duration_list));

		enabled = TRUE;
	}
	gtk_widget_set_sensitive(button_job_list, enabled);
	gtk_widget_set_sensitive(button_referential, enabled);

}
/****************************************************************************/
/* this function is called when a task is selected */
void nc81_sta_task_list_select_child(GtkWidget* gtk_list, GtkWidget *list_item, 
		gpointer data)
{
	gchar* str;
	GtkWidget* label;
	label = g_list_nth_data(
			gtk_container_children(GTK_CONTAINER(list_item) ), 0);
	gtk_label_get(GTK_LABEL(label), &str);
	nc81_sta_task_display(str);
	/* TODO g_free(str)? */
}
/****************************************************************************/
/* this function is called when a task is unselected */
void nc81_sta_task_list_unselect_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	/* do not nc81_sta_display(NULL)! */
}
/****************************************************************************/
/* this function is called for every line to add in the task list */
static void list_fill_callback(gchar** fields, void* data)
{
	gchar* str;
	GList *item_list;
	GtkWidget *item;
	str = g_strjoinv(" ", fields);
	/* create a list item */
	item = gtk_list_item_new_with_label(str);
	gtk_widget_show(GTK_WIDGET(item));
	/* add this item in the list */
	item_list = g_list_append(NULL, item); 
	gtk_list_append_items(GTK_LIST(sta_task_list), item_list);
	/* select the task*/
	if (nc81_task_name != NULL && 
			strcmp(fields[0], nc81_task_name) == 0)
		gtk_list_select_child(GTK_LIST(sta_task_list), item);
	g_free(str);
}

/****************************************************************************/
/* this function updates the task list content, and the task fields */
static void nc81_sta_task_view_update(void)
{
	gtk_list_clear_items(GTK_LIST(sta_task_list),0,-1);
	nc81_sta_task_display(nc81_task_name);
	nc81_command_send("stl \"%r\"", list_fill_callback, 
			NULL, NULL, NULL,NULL);
}
/****************************************************************************/
static gint button_ref_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_sta_ref_view);
	return 0;
}
/****************************************************************************/
static gint button_job_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_sta_job_view);
	return 0;
}
/****************************************************************************/
static gint button_referential_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_task_view);
	return 0;
}
/****************************************************************************/
/* this function creates a widget which contains a list of all
 * statistics tasks, and the task fields
 */
GtkWidget* nc81_sta_task_view_new(void)
{
	GtkWidget *container;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;
	gchar* titles[3];

	container = gtk_vbox_new(FALSE,0);
	vbox1 = gtk_vbox_new(FALSE, 1);
	vbox2 = gtk_vbox_new(FALSE, 2);
	nc81_paned_new(vbox1, vbox2, container);

	/* statistics name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);
	nc81_field_with_icon_new_pack("Referential", widget, 
			GTK_BOX(vbox1), nc81_icon_referential);

	/* construct the task list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 1);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_label_new("Task list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_list_new();
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	sta_task_list = widget;

	gtk_signal_connect(GTK_OBJECT(sta_task_list),
			"select_child",
			nc81_sta_task_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(sta_task_list),
			"unselect_child",
			nc81_sta_task_list_unselect_child, NULL);

	/* construct task fields */
	field_average_duration= gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_average_duration), FALSE);
	nc81_field_with_icon_new_pack("Average duration", 
			field_average_duration, 
			GTK_BOX(vbox2), nc81_icon_none);

	field_mini_duration= gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_mini_duration), FALSE);
	nc81_field_with_icon_new_pack("Minimum duration", 
			field_mini_duration, 
			GTK_BOX(vbox2), nc81_icon_none);

	field_maxi_duration= gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_maxi_duration), FALSE);
	nc81_field_with_icon_new_pack("Maximum duration", 
			field_maxi_duration, 
			GTK_BOX(vbox2), nc81_icon_none);

	/* create the time list */
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox2), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	titles[0] = "Duration";
	titles[1] = "Date";
	widget = gtk_clist_new_with_titles(2,titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 60);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 150);
	gtk_clist_set_column_justification(GTK_CLIST(widget),
			0, GTK_JUSTIFY_RIGHT);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	duration_list = widget;

	/* create the edit menu */
	menu = gtk_menu_new();

	nc81_main_set_edit_menu(GTK_MENU(menu));


	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Referential");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_ref_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);
	
	menu_item = gtk_menu_item_new_with_label("Jobs");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_job_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_job_list = menu_item;

	menu_item = gtk_menu_item_new_with_label("Referential task");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_referential_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_referential = menu_item;
	


	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_sta_task_view_update();
	return container;
}
/****************************************************************************/
