/* file tar_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* tar_list;
static GtkWidget* field_name;
static GtkWidget* field_description;
static GtkWidget* field_hostname;
static GtkWidget* field_ip_port;
static GtkWidget* field_opened;

static GtkWidget* button_edit;
static GtkWidget* button_delete;
static GtkWidget* button_queue_list;

/****************************************************************************/
/* this callback function is called to field the field values */
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Target name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);
	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_description), fields[1]);
	if (strcmp(fields[0], "Host Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_hostname), fields[1]);
	if (strcmp(fields[0], "IP Port") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_ip_port), fields[1]);

	if (strcmp(fields[0], "Opened") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_opened), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);

}


/****************************************************************************/
/* this function intializes the target fields
 * and copies the name of the selected target in the global variable
 */
static void nc81_tar_display(gchar* target_name)
{
	gboolean enabled;

	nc81_target_select(target_name);

	if (nc81_target_name == NULL)
	{
		/* no target selected : blank fields */
		gtk_entry_set_text(GTK_ENTRY(field_name), "");
		gtk_entry_set_text(GTK_ENTRY(field_description), "");
		gtk_entry_set_text(GTK_ENTRY(field_hostname), "");
		gtk_entry_set_text(GTK_ENTRY(field_ip_port), "");
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_opened), FALSE);
		enabled = FALSE;
	}
	else
	{
		nc81_command_send("ti \"%T\"", field_fill_callback,
				NULL,NULL,NULL,NULL);
		enabled = TRUE;
	}
	gtk_widget_set_sensitive(button_edit, enabled);
	gtk_widget_set_sensitive(button_delete, enabled);
	gtk_widget_set_sensitive(button_queue_list, enabled);

}
/****************************************************************************/
/* this function is called when a target is selected */
static void nc81_tar_list_select_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	gchar* str;
	GtkWidget* label;
	label = g_list_nth_data(
			gtk_container_children(GTK_CONTAINER(list_item) ), 0);
	gtk_label_get(GTK_LABEL(label), &str);
	nc81_tar_display(str);
	/* TODO g_free(str)? */
}
/****************************************************************************/
/* this function is called when a target is unselected */
static void nc81_tar_list_unselect_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	/* do not nc81_tar_display(NULL)! */
}
/****************************************************************************/
/* this function is called for every line to add in the target list */
static void list_fill_callback(gchar** fields, void* data)
{
	gchar* str;
	GList *item_list;
	GtkWidget *item;
	str = g_strjoinv(" ", fields);
	/* create a list item */
	item = gtk_list_item_new_with_label(str);
	gtk_widget_show(GTK_WIDGET(item));
	/* add this item in the list */
	item_list = g_list_append(NULL, item); 
	gtk_list_append_items(GTK_LIST(tar_list), item_list);
	/* select the target */
	if (nc81_target_name != NULL && strcmp(fields[0], nc81_target_name) == 0)
		gtk_list_select_child(GTK_LIST(tar_list), item);
	g_free(str);
}

/****************************************************************************/
/* this function updates the target list content, and
 * the target fields */
static void nc81_tar_view_update(void)
{
	gtk_list_clear_items(GTK_LIST(tar_list),0,-1);
	nc81_tar_display(nc81_target_name);
	nc81_command_send("tl", list_fill_callback, NULL, NULL, NULL, NULL);
}

/****************************************************************************/
static gint button_edit_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_target_edit);
	return 0;
}
/****************************************************************************/
static gint button_new_clicked(GtkWidget *widget, gpointer data)
{
	nc81_target_select(NULL);
	nc81_main_load(nc81_form_target_edit);
	return 0;
}
/****************************************************************************/
static gint button_delete_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_target_delete);
	return 0;
}
/****************************************************************************/
static gint button_queue_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_queue_view);
	return 0;
}
/****************************************************************************/
/*
 * this function creates a widget which contains a list of all
 * targets, and the target fields
 */
GtkWidget* nc81_target_view_new(void)
{
	GtkWidget *container;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;

	container = gtk_vbox_new(FALSE, 2);
	vbox1 = gtk_vbox_new(FALSE, 1);
	vbox2 = gtk_vbox_new(FALSE, 2);
	nc81_paned_new(vbox1, vbox2, container);

	/* construct the target list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 1);

	/* TODO change icon */
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_referential], 
			nc81_icon_bitmap_array[nc81_icon_referential]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_label_new("Target list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_list_new();
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	tar_list = widget;

	gtk_signal_connect(GTK_OBJECT(tar_list),
			"select_child",
			nc81_tar_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(tar_list),
			"unselect_child",
			nc81_tar_list_unselect_child, NULL);

	/* construct target fields */
	field_name = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_name), FALSE);
	nc81_field_with_icon_new_pack("Target", field_name, 
			GTK_BOX(vbox2), nc81_icon_none);

	field_description = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_description), FALSE);
	nc81_field_with_icon_new_pack("Description", field_description, 
			GTK_BOX(vbox2), nc81_icon_none);

	field_hostname = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_hostname), FALSE);
	nc81_field_with_icon_new_pack("Host name", field_hostname, 
			GTK_BOX(vbox2), nc81_icon_none);

	field_ip_port= gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_ip_port), FALSE);
	nc81_field_with_icon_new_pack("IP port", field_ip_port, 
			GTK_BOX(vbox2), nc81_icon_none);

	widget = gtk_check_button_new_with_label("Opened");
	gtk_box_pack_start(GTK_BOX(vbox2), widget, FALSE, TRUE, 1);
	gtk_widget_set_sensitive(widget, FALSE);
	field_opened = widget;

	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Edit");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_edit_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_edit = menu_item;

	menu_item = gtk_menu_item_new_with_label("Add");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_new_clicked), NULL);

	menu_item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_delete_clicked), NULL);
	button_delete = menu_item;

	nc81_main_set_edit_menu(GTK_MENU(menu));

	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Queues");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_queue_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_queue_list = menu_item;

	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_tar_view_update();

	return container;
}
/****************************************************************************/
