/* file user_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

#define DEFAULT_TYPE 1
static char* type_array[] = {"none", "consult", "manage", "admin", NULL};
static char* priv_array[] = { 
	"generic", "communication", "user", "engine", "target",
	"referential", "job", "day", "variable", "log", "stat", NULL};
static GtkWidget* priv_field[11];
static GtkWidget *field_name;

/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	int i;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "User Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);

	for (i = 0; priv_array[i] != NULL; i++) {
		if (strcmp(fields[0], priv_array[i]) == 0) {
			gtk_entry_set_text(GTK_ENTRY(
					GTK_COMBO(priv_field[i])->entry), 
					fields[1]);
			break;
		}
	}
	
}
/****************************************************************************/
/* this function updates the user fields */
static void nc81_user_edit_update(void)
{
	if (nc81_user_name == NULL)
		return;
	nc81_command_send("ui \"%u\"", field_fill_callback, 
			NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* user_name;
	GString* priv_list;
	gchar* value;
	gboolean changed;
	int i,j;

	/* user name */
	user_name = gtk_entry_get_text(GTK_ENTRY(field_name));
	if (nc81_user_name == NULL)
	{
		/* create a new user */
		nc81_command_send( "ua \"%x\"",  
			field_valid_callback,
			NULL, user_name, NULL, NULL);
		nc81_user_select(user_name);
	}
	else
	{
		changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
					GTK_OBJECT(field_name)));
		if (changed == TRUE)
		{
			/* change the name of the user */
			/* TODO
			nc81_command_send("rsn \"%r\" \"%x\"", 
				field_valid_callback,
				NULL, user_name, NULL, NULL);
				*/
			nc81_user_select(user_name);
			
		}
	}

	/* for each type of privilege : none/consult/manage/admin */
	for (i = 0; type_array[i] != NULL; i++)
	{
		priv_list = g_string_new("");
		/* for each privilege */
		for (j=0; priv_array[j]!=NULL; j++)
		{
			value = gtk_entry_get_text(GTK_ENTRY(
					GTK_COMBO(priv_field[j])->entry));
			if (strcmp(type_array[i], value) != 0)
				continue;
			g_string_append(priv_list, priv_array[j]);
			g_string_append_c(priv_list, ' ');
		}
		if (strcmp(priv_list->str, "") == 0)
			continue;
		/* create the 'usp' command */
		value = g_strdup_printf("usp \"%s\" %s %s\n", 
				nc81_user_name,
				type_array[i], 
				priv_list->str);
		g_string_free(priv_list, TRUE);
		nc81_command_send(value,
			field_valid_callback,
			NULL, NULL, NULL, NULL);
		g_free(value);
	}

	/* TODO do not close if error */
	nc81_main_load(nc81_form_user_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_user_view);
	return 0;
}
/****************************************************************************/
static void nc81_field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}
/****************************************************************************/
/* this function creates a widget which edit or add a user*/
GtkWidget* nc81_user_edit_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *table;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *frame;
	GList *list;
	int i,j;
	vbox = gtk_vbox_new(FALSE, 2);

	/* user name field */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);
	widget = gtk_label_new((nc81_user_name==NULL)?
			"New user": "User");
	if (nc81_user_name != NULL)
		gtk_widget_set_sensitive(GTK_WIDGET(widget), FALSE);

	gtk_misc_set_alignment(GTK_MISC(widget), 0 , 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	gtk_object_set_user_data(GTK_OBJECT(widget), GUINT_TO_POINTER(FALSE));
	gtk_widget_grab_focus(widget);
	field_name = widget;


	/* construct user privileges */
	frame = gtk_frame_new("Privileges");
	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 1);
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_container_add(GTK_CONTAINER(frame), scrolled);
	gtk_container_set_border_width(GTK_CONTAINER(scrolled), 5);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	table = gtk_table_new(11, 2, FALSE);
	gtk_container_set_border_width(GTK_CONTAINER(table), 2);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			table);

	/* create user privileges fields */
	for (i=0; priv_array[i]!=NULL; i++)
	{
		widget = gtk_label_new(priv_array[i]);
		gtk_misc_set_alignment(GTK_MISC(widget), 0 , 0.5);
		gtk_table_attach_defaults(GTK_TABLE(table), widget, 
				0, 1, i * 2, i * 2 + 1);
		widget = gtk_combo_new();
		gtk_table_attach_defaults(GTK_TABLE(table), widget, 
				1, 2, i * 2, i * 2 + 1);
		list = NULL;
		for (j = 0; type_array[j] != NULL; j++)
			list = g_list_append(list, type_array[j]);
		gtk_combo_set_popdown_strings(GTK_COMBO(widget), list);
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(widget)->entry),
				type_array[DEFAULT_TYPE]);
		g_list_free(list);
		priv_field[i] = widget;

	}
	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	nc81_user_edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_name), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	return vbox;
}
