/* file var_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget *field_name;
static GtkWidget *field_value;

/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Variable name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);
	
	if (strcmp(fields[0], "Value") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_value), fields[1]);
}
/****************************************************************************/
/* this function updates the variable fields */
static void nc81_var_edit_update(void)
{
	if (nc81_variable_name == NULL)
		return;
	nc81_command_send("vi \"%v\"", field_fill_callback, 
			NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* variable_name;
	gboolean changed;
	gchar* value;

	/* variable name */
	variable_name = gtk_entry_get_text(GTK_ENTRY(field_name));
	if (nc81_variable_name == NULL)
	{
		/* create a new variable */
		nc81_command_send( "va \"%x\"",  
			field_name_valid_callback,
			NULL, variable_name, NULL, NULL);
		nc81_variable_select(variable_name);
	}

	/* auto planed / auto purged */
	changed = GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_value)));
	if (changed == TRUE)
	{
		value = gtk_entry_get_text(GTK_ENTRY(field_value));
		nc81_command_send("vsv \"%v\" \"%x\"", 
			field_name_valid_callback, NULL, 
			value, NULL, NULL);
	}

	/* TODO do not close if error */
	nc81_main_load(nc81_form_variable_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_variable_view);
	return 0;
}
/****************************************************************************/
static void nc81_field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}

/****************************************************************************/
/* this function creates a widget which edit or add a variable*/
GtkWidget* nc81_var_edit_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	vbox = gtk_vbox_new(FALSE, 2);

	/* variable name field */
	field_name = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_name), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack((nc81_variable_name==NULL)? 
				"New variable": "Variable",
			field_name, GTK_BOX(vbox));

	/* variable value field */
	field_value = gtk_entry_new();
	gtk_object_set_user_data(GTK_OBJECT(field_value), 
			GUINT_TO_POINTER(FALSE));
	nc81_field_new_pack("Value", field_value, GTK_BOX(vbox));

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	nc81_var_edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_name), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_value), "changed",
			GTK_SIGNAL_FUNC(nc81_field_changed_callback), NULL);
	return vbox;
}
/****************************************************************************/
