/* file exp_cond_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

GtkWidget *condition_list;
gint condition_index;

/****************************************************************************/
static void condition_select(gint index)
{
	condition_index = index;
}
/****************************************************************************/
/* this function is called for every line to add in the condition list */
static void list_fill_callback(gchar** fields, void* data)
{
	gint row;
	gint i;
	gchar* values[4];
	/* initialize default value for each cell */
	for (i = 0; i < 4; i++)
		values[i] = "";
	/* copy fields values */
	for (i = 0; fields[i] != NULL; i++)
		values[i] = fields[i];
	row = gtk_clist_append(GTK_CLIST(condition_list), values);
}
/****************************************************************************/
/* this function is called when an condition is selected */
static void nc81_condition_list_select(GtkWidget* gtk_clist, 
		gint row, gint column, GdkEvent *event, gpointer data)
{
	condition_select(row);
}
/****************************************************************************/
/* this function is called when a condition is unselected */
static void nc81_condition_list_unselect(GtkWidget* gtk_clist, 
		gint row, gint column, GdkEvent *event, gpointer data)
{
	condition_select(-1);
}
/****************************************************************************/
static void condition_list_init(void)
{
	condition_select(-1);
	gtk_clist_freeze(GTK_CLIST(condition_list));
	gtk_clist_clear(GTK_CLIST(condition_list));
	nc81_command_send("xtcl -s \"%d\" \"%t\"",
			list_fill_callback, NULL, NULL, NULL,NULL);
	gtk_clist_thaw(GTK_CLIST(condition_list));
}
/****************************************************************************/
static gint button_validate_clicked(GtkWidget *widget, gpointer data)
{
	/*
	gchar* str;
	if (condition_index == -1)
		return 0;
	str = g_strdup_printf("%i", condition_index + 1);
	nc81_command_send("xtcv \"%d\" \"%t\" %x",
			list_fill_callback, NULL, str, NULL, NULL);
	g_free(str);
	condition_list_init();
	*/
	return 0;
}
/****************************************************************************/
static gint button_update_clicked(GtkWidget *widget, gpointer data)
{
	condition_list_init();
	return 0;
}
/****************************************************************************/
static gint button_exp_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_view);
	return 0;
}
/****************************************************************************/
static gint button_parent_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_task_view);
	return 0;
}

/****************************************************************************/
/* this function creates a widget to edit task conditions */
GtkWidget* nc81_exp_task_cond_view_new()
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;
	gchar* titles[4];

	assert(nc81_day_name != NULL && nc81_task_name != NULL);

	vbox = gtk_vbox_new(FALSE, 1);

	/* exploitation name */
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE,1);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_day], 
			nc81_icon_bitmap_array[nc81_icon_day]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_label_new("Day");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 1);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_day_name);

	/* task name */
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_label_new("Task");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 1);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_task_name);

	/* condition list */
	widget = gtk_label_new("Condition list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 1);
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	titles[0] = "Type";
	titles[1] = "Argument 1";
	titles[2] = "Argument 2";
	titles[3] = "State";
	widget = gtk_clist_new_with_titles(4,titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 70);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 90);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled), widget);
	condition_list = widget;

	gtk_signal_connect(GTK_OBJECT(condition_list), "select_row",
			nc81_condition_list_select, NULL);
	gtk_signal_connect(GTK_OBJECT(condition_list), "unselect_row",
			nc81_condition_list_unselect, NULL);
	
	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Validate");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_validate_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Update");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_update_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	nc81_main_set_edit_menu(GTK_MENU(menu));

	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Day");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_exp_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Task");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_parent_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	nc81_main_set_select_menu(GTK_MENU(menu));

	/* init condition list */
	condition_list_init();

	return vbox;
}
/****************************************************************************/
