/* file exp_day_edit.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget *field_name;
static GtkWidget *field_autopurged;
static GtkWidget *field_enabled;
static GtkWidget *field_start_time;
static GtkWidget *field_referential;
/****************************************************************************/
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Day Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_name), fields[1]);

	if (strcmp(fields[0], "Enabled") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_enabled), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);
	
	if (strcmp(fields[0], "Auto Purged") == 0)
		gtk_toggle_button_set_active(
				GTK_TOGGLE_BUTTON(field_autopurged), 
				(strcmp(fields[1], "true") == 0)?TRUE:FALSE);
	
	if (strcmp(fields[0], "Start Time") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_start_time), fields[1]);

	if (strcmp(fields[0], "Referential") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_referential), fields[1]);

}
/****************************************************************************/
/* this function updates the day fields */
static void edit_update(void)
{
	if (nc81_day_name == NULL)
		return;
	nc81_command_send("xdi \"%d\"", field_fill_callback, 
			NULL, NULL, NULL,NULL);
}

/****************************************************************************/
static void field_name_valid_callback(gchar** fields, void* data)
{
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gboolean enabled;

	/* field enabled */
	if (GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_enabled))) == TRUE)
	{
		enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_enabled));
		nc81_command_send("xdse \"%d\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(enabled == TRUE)?"true":"false",
			NULL, NULL);
	}


	/* auto purged */
	if ((GPOINTER_TO_UINT(gtk_object_get_user_data(
				GTK_OBJECT(field_autopurged)))) == TRUE)
	{
		enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
					field_autopurged));
		nc81_command_send("xdsa \"%d\" \"%x\"", 
			field_name_valid_callback, NULL, 
			(enabled == TRUE)?"true":"false",
			NULL, NULL);
	}

	/* TODO do not close if error */
	nc81_main_load(nc81_form_exploitation_view);
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_view);
	return 0;
}
/****************************************************************************/
static void field_changed_callback(GtkWidget *field, gpointer data)
{
	/* memorize that the field value was changed */
	gtk_object_set_user_data(GTK_OBJECT(field), GUINT_TO_POINTER(TRUE));
}

/****************************************************************************/
/* this function creates a widget which edit or add a referential*/
GtkWidget* nc81_day_edit_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	vbox = gtk_vbox_new(FALSE, 2);

	/* day name field */
	field_name = gtk_entry_new();
	gtk_widget_set_sensitive(field_name, FALSE);
	nc81_field_new_pack("Day name", field_name, GTK_BOX(vbox));

	/* enabled field */
	widget = gtk_check_button_new_with_label("Enabled");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_enabled = widget;

	/* auto purged field */
	widget = gtk_check_button_new_with_label("Auto purged");
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, TRUE, 1);
	field_autopurged = widget;

	/* referential name field */
	field_referential = gtk_entry_new();
	gtk_widget_set_sensitive(field_referential, FALSE);
	nc81_field_new_pack("Referential", field_referential, GTK_BOX(vbox));

	/* start time field */
	field_start_time = gtk_entry_new();
	gtk_widget_set_sensitive(field_start_time, FALSE);
	nc81_field_new_pack("Start time", field_start_time, GTK_BOX(vbox));

	/* button box */
	hbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(hbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_end(GTK_BOX(vbox), hbox, FALSE, TRUE, 1);

	/* OK button */
	widget = gtk_button_new_with_label("OK");
	/*
	GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(widget);
	*/
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* Cancel button */
	widget = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(hbox), widget);

	/* initialize fields */
	edit_update();

	/* connect the changed signals */
	gtk_signal_connect(GTK_OBJECT(field_enabled), "clicked",
			GTK_SIGNAL_FUNC(field_changed_callback), NULL);
	gtk_signal_connect(GTK_OBJECT(field_autopurged), "clicked",
			GTK_SIGNAL_FUNC(field_changed_callback), NULL);
	return vbox;
}
/****************************************************************************/
