/* file exp_dep_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* dep_list;
static GtkWidget* day_name;
static GtkWidget* task_name;
static GtkWidget* job_name;
static gboolean show_job_dep;
gint dep_index;

static void dep_list_init(void);
static void list_fill_callback(gchar** fields, void* data);
/****************************************************************************/
static void dep_select(gint index)
{
	dep_index = index;
	if (dep_index == -1)
		return;
	if (gtk_clist_get_selectable(GTK_CLIST(dep_list), dep_index) == FALSE)
		dep_index = -1;
}
/****************************************************************************/
/* this function is called when a dependence is selected */
static void dep_list_select(GtkWidget* gtk_clist, 
		gint row, gint column, GdkEvent *event, gpointer data)
{
	dep_select(row);
}
/****************************************************************************/
/* this function is called when a dependence is unselected */
static void dep_list_unselect(GtkWidget* gtk_clist, 
		gint row, gint column, GdkEvent *event, gpointer data)
{
	dep_select(-1);
}
/****************************************************************************/
static gint button_validate_clicked(GtkWidget *widget, gpointer data)
{
	gchar* upstream;
	gchar* downstream;
	gchar* deptype;
	gchar* cmd;
	if (dep_index == -1)
		return 0;
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 0, &upstream);
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 1, &downstream);
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 2, &deptype);
	if (show_job_dep == TRUE) 
		cmd = "xjdu \"%d\" \"%t\" \"%x\" \"%y\" \"%z\" validated";
	else
		cmd = "xtdu \"%d\" \"%x\" \"%y\" \"%z\" validated";
	nc81_command_send(cmd, list_fill_callback, NULL, 
				upstream, downstream, deptype);
	dep_list_init();
	return 0;
}
/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	gchar* upstream;
	gchar* downstream;
	gchar* deptype;
	gchar* cmd;
	if (dep_index == -1)
		return 0;
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 0, &upstream);
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 1, &downstream);
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 2, &deptype);
	if (show_job_dep == TRUE) 
		cmd = "xjdu \"%d\" \"%t\" \"%x\" \"%y\" \"%z\" canceled";
	else
		cmd = "xtdu \"%d\" \"%x\" \"%y\" \"%z\" canceled";
	nc81_command_send(cmd, list_fill_callback, NULL, 
				upstream, downstream, deptype);
	dep_list_init();
	return 0;
}
/****************************************************************************/
static gint button_day_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_view);
	return 0;
}
/****************************************************************************/
static gint button_task_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_task_view);
	return 0;
}
/****************************************************************************/
static gint button_job_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_job_view);
	return 0;
}
/****************************************************************************/
static gint button_follow_clicked(GtkWidget *widget, gpointer data)
{
	gchar* upstream;
	gchar* downstream;
	if (dep_index == -1)
		return 0;
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 0, &upstream);
	gtk_clist_get_text(GTK_CLIST(dep_list), dep_index, 1, &downstream);
	if (show_job_dep == TRUE)
	{
		if (strcmp(upstream, nc81_job_name) == 0)
			nc81_job_select(downstream);
		else
			nc81_job_select(upstream);
	} else {
		if (strcmp(upstream, nc81_task_name) == 0)
			nc81_task_select(downstream);
		else
			nc81_task_select(upstream);
	}
	dep_list_init();
	return 0;
}
/****************************************************************************/
/* this function is called for every line to add in the dependence list */
static void list_fill_callback(gchar** fields, void* data)
{
	gint row;
	gint i;
	gchar* values[4];
	/* initialize default value for each cell */
	for (i = 0; i < 4; i++)
		values[i] = "";
	/* copy fields values */
	for (i = 0; fields[i] != NULL; i++)
		values[i] = fields[i];
	row = gtk_clist_append(GTK_CLIST(dep_list), values);
}
/****************************************************************************/
static void dep_list_init(void)
{
	GdkColor foreground, background;
	gint row;
	gchar* values[4];

	dep_select(-1);

	gtk_entry_set_text(GTK_ENTRY(day_name), nc81_day_name);
	gtk_entry_set_text(GTK_ENTRY(task_name), nc81_task_name);
	if (show_job_dep == TRUE)
		gtk_entry_set_text(GTK_ENTRY(job_name), nc81_job_name);

	gtk_clist_freeze(GTK_CLIST(dep_list));
	gtk_clist_clear(GTK_CLIST(dep_list));
	background = GTK_WIDGET(dep_list)->style->fg[0];
	foreground = GTK_WIDGET(dep_list)->style->bg[0];
	/* add upstream tasks */
	values[0] = "Upstream";
	values[1] = "";
	values[2] = "";
	values[3] = "";
	row = gtk_clist_append(GTK_CLIST(dep_list), values);
	gtk_clist_set_selectable(GTK_CLIST(dep_list), row, FALSE);
	gtk_clist_set_foreground(GTK_CLIST(dep_list), row, &foreground);
	gtk_clist_set_background(GTK_CLIST(dep_list), row, &background);
	nc81_command_send(
			(show_job_dep == TRUE) ? 
				"xjdl -ts \"%d\" \"%t\" -b \"%j\"":
				"xtdl -ts \"%d\" -b \"%t\"", 
			list_fill_callback, NULL, NULL, NULL,NULL);
	/* add downstream tasks */
	values[0] = "Downstream";
	values[1] = "";
	values[2] = "";
	values[3] = "";
	row = gtk_clist_append(GTK_CLIST(dep_list), values);
	gtk_clist_set_selectable(GTK_CLIST(dep_list), row, FALSE);
	gtk_clist_set_foreground(GTK_CLIST(dep_list), row, &foreground);
	gtk_clist_set_background(GTK_CLIST(dep_list), row, &background);
	nc81_command_send(
			(show_job_dep == TRUE) ? 
				"xjdl -ts \"%d\" \"%t\" -a \"%j\"":
				"xtdl -ts \"%d\" -a \"%t\"", 
			list_fill_callback, NULL, NULL, NULL,NULL);
	gtk_clist_thaw(GTK_CLIST(dep_list));
}
/****************************************************************************/
/* this function creates a widget which contains a list of all
 * dependencies
 */
static GtkWidget* nc81_exp_dep_view_new(gboolean job_dep)
{
	GtkWidget *container;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;
	gchar* titles[] = { "Upstream", "Downstream", "Type", "State" };

	show_job_dep = job_dep;
	container = gtk_vbox_new(FALSE,0);

	/* day name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, FALSE, 1);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_day], 
			nc81_icon_bitmap_array[nc81_icon_day]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_label_new("Day");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(container), widget, FALSE, FALSE, 0);
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	day_name = widget;

	/* task name */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, FALSE, 1);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	widget = gtk_label_new("Task");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(container), widget, FALSE, FALSE, 0);
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	task_name = widget;

	/* job name */
	job_name = NULL;
	if (show_job_dep == TRUE)
	{
		hbox = gtk_hbox_new(FALSE, 2);
		gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, FALSE, 1);
		widget = gtk_pixmap_new(
				nc81_icon_pixmap_array[nc81_icon_job], 
				nc81_icon_bitmap_array[nc81_icon_job]);
		gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
		widget = gtk_label_new("Job");
		gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
		gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
		widget = gtk_entry_new();
		gtk_box_pack_start(GTK_BOX(container), widget, FALSE, FALSE, 0);
		gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
		job_name = widget;
	}

	/* construct the dependence list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 1);
	gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, TRUE, 1);

	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_label_new("Dependencies");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(container), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_clist_new_with_titles(4, titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 90);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 90);
	gtk_clist_set_column_width(GTK_CLIST(widget), 2, 70);
	gtk_clist_set_column_width(GTK_CLIST(widget), 3, 70);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	dep_list = widget;

	gtk_signal_connect(GTK_OBJECT(dep_list),
			"select_row", dep_list_select, NULL);
	gtk_signal_connect(GTK_OBJECT(dep_list),
			"unselect_row", dep_list_unselect, NULL);

	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Validate dependence");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_validate_clicked), NULL);

	menu_item = gtk_menu_item_new_with_label("Cancel dependence");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);

	nc81_main_set_edit_menu(GTK_MENU(menu));

	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Day");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_day_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Task");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_task_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	if (show_job_dep == TRUE)
	{
		menu_item = gtk_menu_item_new_with_label("Job");
		gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
				GTK_SIGNAL_FUNC(button_job_clicked), NULL);
		gtk_menu_append(GTK_MENU(menu), menu_item);
	}

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Follow");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_follow_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	nc81_main_set_select_menu(GTK_MENU(menu));

	dep_list_init();
	return container;
}
/****************************************************************************/
GtkWidget* nc81_exp_task_dep_view_new()
{
	return nc81_exp_dep_view_new(FALSE);
}
/****************************************************************************/
GtkWidget* nc81_exp_job_dep_view_new()
{
	return nc81_exp_dep_view_new(TRUE);
}
/****************************************************************************/
