/* file exp_job_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* exp_job_list;
static GtkWidget* field_job_name;
static GtkWidget* field_job_description;
static GtkWidget* field_job_path;
static GtkWidget* field_job_argument;

static GtkWidget* button_edit;
static GtkWidget* button_log;
static GtkWidget* button_dep_list;

/****************************************************************************/
/* this callback function is called to field the field values */
static void field_fill_callback(gchar** fields, void* data)
{
	gchar* c;
	/* remove the last character of the field name (:) */
	if ((c = strrchr(fields[0], ':')) != NULL)
		*c = '\0';

	/* g_print("[%s] - [%s]\n", fields[0], fields[1]); */

	if (strcmp(fields[0], "Job Name") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_name), fields[1]);

	if (strcmp(fields[0], "Description") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_description), fields[1]);

	if (strcmp(fields[0], "Path") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_path), fields[1]);

	if (strcmp(fields[0], "Argument") == 0)
		gtk_entry_set_text(GTK_ENTRY(field_job_argument), fields[1]);

}


/****************************************************************************/
/* this function intializes the job fields
 * and copies the name of the selected job in the global variable
 */
void nc81_exp_job_display(gchar* job_name)
{
	gboolean enabled;

	nc81_job_select(job_name);

	if (nc81_day_name == NULL || nc81_task_name == NULL || nc81_job_name == NULL)
	{
		/* no exploitation job selected : blank fields */
		gtk_entry_set_text(GTK_ENTRY(field_job_name), "");
		gtk_entry_set_text(GTK_ENTRY(field_job_description), "");
		gtk_entry_set_text(GTK_ENTRY(field_job_path), "");
		gtk_entry_set_text(GTK_ENTRY(field_job_argument), "");
		enabled = FALSE;
	}
	else
	{
		nc81_command_send("xji \"%d\" \"%t\" \"%j\"", 
				field_fill_callback, NULL,NULL,NULL, NULL);
		enabled = TRUE;
	}
	gtk_widget_set_sensitive(button_edit, enabled);
	gtk_widget_set_sensitive(button_log, enabled);
	gtk_widget_set_sensitive(button_dep_list, enabled);
}
/****************************************************************************/
/* this function is called when a job is selected */
void nc81_exp_job_list_select_child(GtkWidget* gtk_clist, gint row, 
		gint column, GdkEvent *event, gpointer data)
{
	gchar* str;
	/* row 0 is the job name */
	gtk_clist_get_text(GTK_CLIST(exp_job_list), row, 0 ,&str);
	nc81_exp_job_display(str);
}
/****************************************************************************/
/* this function is called when a job is unselected */
void nc81_exp_job_list_unselect_child(GtkWidget* gtk_clist, gint row, 
		gint column, GdkEvent *event, gpointer data)
{
	/* do not nc81_exp_display(NULL)! */
}
/****************************************************************************/
/* this function is called for every line to add in the job list */
static void list_fill_callback(gchar** fields, void* data)
{
	gint row;
	gchar* values[3];
	values[0] = fields[0]; /* name */
	values[1] = fields[1]; /* state */
	values[2] = fields[2]; /* description */
	row = gtk_clist_append(GTK_CLIST(exp_job_list), values);
	/* select the job*/
	if (nc81_job_name != NULL && 
			strcmp(fields[0], nc81_job_name) == 0)
		gtk_clist_select_row(GTK_CLIST(exp_job_list), row, 0);
}

/****************************************************************************/
/* this function updates the job list content, and the job fields */
void nc81_exp_job_view_update(void)
{
	gtk_clist_freeze(GTK_CLIST(exp_job_list));
	gtk_clist_clear(GTK_CLIST(exp_job_list));
	nc81_exp_job_display(nc81_job_name);
	nc81_command_send("xjl -ds \"%d\" \"%t\"", list_fill_callback, 
			NULL, NULL, NULL,NULL);
	gtk_clist_thaw(GTK_CLIST(exp_job_list));
}

/****************************************************************************/
static gint button_edit_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_job_edit);
	return 0;
}
/****************************************************************************/
static gint button_update_clicked(GtkWidget *widget, gpointer data)
{
	nc81_exp_job_view_update();
	return 0;
}
/****************************************************************************/
static gint button_day_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_view);
	return 0;
}
/****************************************************************************/
static gint button_task_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_task_view);
	return 0;
}
/****************************************************************************/
static gint button_log_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_log_job_view);
	return 0;
}
/****************************************************************************/
static gint button_dep_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_job_dep_view);
	return 0;
}
/****************************************************************************/
/* this function creates a widget which contains a list of all
 * exploitation jobs, and the job fields
 */
GtkWidget* nc81_exp_job_view_new(void)
{
	GtkWidget *container;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;
	gchar* titles[3];

	container = gtk_vbox_new(FALSE,0);
	vbox1 = gtk_vbox_new(FALSE, 1);
	vbox2 = gtk_vbox_new(FALSE, 2);
	nc81_paned_new(vbox1, vbox2, container);

	/* day name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_day_name);
	nc81_field_with_icon_new_pack("Day", widget, 
			GTK_BOX(vbox1), nc81_icon_day);

	/* task name */
	/*
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_task], 
			nc81_icon_bitmap_array[nc81_icon_task]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
	widget = gtk_label_new("Task");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 0);
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_task_name);
	*/
	/* task name */
	widget = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_task_name);
	nc81_field_with_icon_new_pack("Task", widget, 
			GTK_BOX(vbox1), nc81_icon_task);

	/* construct the job list */
	hbox = gtk_hbox_new(FALSE, 1);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 0);
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_job], 
			nc81_icon_bitmap_array[nc81_icon_job]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 0);
	widget = gtk_label_new("Job list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 0);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	titles[0] = "Name";
	titles[1] = "State";
	titles[2] = "Description";
	widget = gtk_clist_new_with_titles(3,titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 90);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 70);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	exp_job_list = widget;

	gtk_signal_connect(GTK_OBJECT(exp_job_list), "select_row",
			nc81_exp_job_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(exp_job_list), "unselect_row",
			nc81_exp_job_list_unselect_child, NULL);

	/* construct task fields */

	/* job name */
	field_job_name = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_name), FALSE);
	nc81_field_new_pack("Job name", field_job_name, GTK_BOX(vbox2));

	/* description */
	field_job_description = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_description), FALSE);
	nc81_field_new_pack("Description", field_job_description, GTK_BOX(vbox2));

	/* path */
	field_job_path = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_path), FALSE);
	nc81_field_new_pack("Path", field_job_path, GTK_BOX(vbox2));

	/* argument */
	field_job_argument = gtk_entry_new();
	gtk_entry_set_editable(GTK_ENTRY(field_job_argument), FALSE);
	nc81_field_new_pack("Argument", field_job_argument, GTK_BOX(vbox2));

	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Edit");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_edit_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_edit = menu_item;

	menu_item = gtk_menu_item_new_with_label("Update");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_update_clicked), NULL);

	nc81_main_set_edit_menu(GTK_MENU(menu));

	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Day");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_day_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Task");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_task_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Job dependencies");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_dep_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_dep_list = menu_item;


	menu_item = gtk_menu_item_new_with_label("Log Entries");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_log_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_log = menu_item;

	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_exp_job_view_update();
	return container;
}
/****************************************************************************/
