/* file log_day_view.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include "global.h"
#include "command.h"

static GtkWidget* log_day_list;
static GtkWidget* log_entries_list;

static GtkWidget* button_delete;
static GtkWidget* button_task_list;
static GtkWidget* button_exploitation;

/****************************************************************************/
/* this function is called for every line to add in the log entries list */
static void log_entries_list_fill_callback(gchar** fields, void* data)
{
	gint row;
	gchar* values[3];
	values[0] = fields[0]; /* date */
	values[1] = fields[1]; /* type */
	values[2] = fields[2]; /* data */
	row = gtk_clist_append(GTK_CLIST(log_entries_list), values);
}

/****************************************************************************/
/* this function intializes the log entries list 
 * and copies the name of the selected day in the global variable
 */
static void log_entries_list_update(char* day_name)
{

	gboolean enabled;
	nc81_day_select(day_name);
	if (nc81_day_name == NULL)
	{
		enabled = FALSE;
	}
	else
	{
		gtk_clist_freeze(GTK_CLIST(log_entries_list));
		gtk_clist_clear(GTK_CLIST(log_entries_list));
		nc81_command_send("ldel \"%d\"", 
				log_entries_list_fill_callback, 
				NULL, NULL, NULL,NULL);
		gtk_clist_thaw(GTK_CLIST(log_entries_list));

		enabled = TRUE;
	}
	gtk_widget_set_sensitive(button_delete, enabled);
	gtk_widget_set_sensitive(button_task_list, enabled);
	gtk_widget_set_sensitive(button_exploitation, enabled);

}
/****************************************************************************/
/* this function is called when a day is selected */
static void nc81_log_day_list_select_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	gchar* str;
	GtkWidget* label;
	label = g_list_nth_data(
			gtk_container_children(GTK_CONTAINER(list_item) ), 0);
	gtk_label_get(GTK_LABEL(label), &str);
	log_entries_list_update(str);
	/* TODO g_free(str)? */
}
/****************************************************************************/
/* this function is called when a day is unselected */
static void nc81_log_day_list_unselect_child(GtkWidget* gtk_list, 
		GtkWidget *list_item, gpointer data)
{
	/* do not nc81_day_display(NULL)! */
}
/****************************************************************************/
/* this function is called for every line to add in the log day list */
static void list_fill_callback(gchar** fields, void* data)
{
	gchar* str;
	GList *item_list;
	GtkWidget *item;
	str = g_strjoinv(" ", fields);
	/* create a list item */
	item = gtk_list_item_new_with_label(str);
	gtk_widget_show(GTK_WIDGET(item));
	/* add this item in the list */
	item_list = g_list_append(NULL, item); 
	gtk_list_append_items(GTK_LIST(log_day_list), item_list);
	/* select the day */
	if (nc81_day_name != NULL && strcmp(fields[0], nc81_day_name) == 0)
		gtk_list_select_child(GTK_LIST(log_day_list), item);
	g_free(str);
}

/****************************************************************************/
/* this function updates the log day list content, and the log entries list */
static void nc81_day_view_update(void)
{
	gtk_list_clear_items(GTK_LIST(log_day_list),0,-1);
	log_entries_list_update(nc81_day_name);
	nc81_command_send("ldl", list_fill_callback, NULL, NULL, NULL, NULL);
}

/****************************************************************************/
static gint button_delete_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_log_day_delete);
	return 0;
}
/****************************************************************************/
static gint button_task_list_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_log_task_view);
	return 0;
}
/****************************************************************************/
static gint button_exploitation_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_view);
	return 0;
}
/****************************************************************************/
/* this function creates a widget which contains a list of all
 * log days, and log entries 
 */
GtkWidget* nc81_log_day_view_new(void)
{
	GtkWidget *container;
	GtkWidget *vbox1;
	GtkWidget *vbox2;
	GtkWidget *hbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkWidget *menu;
	GtkWidget *menu_item;
	gchar* titles[3];

	container = gtk_vbox_new(FALSE, 2);
	vbox1 = gtk_vbox_new(FALSE, 1);
	vbox2 = gtk_vbox_new(FALSE, 2);
	nc81_paned_new(vbox1, vbox2, container);

	/* construct the log day list */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox1), hbox, FALSE, TRUE, 1);

	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_day], 
			nc81_icon_bitmap_array[nc81_icon_day]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	widget = gtk_label_new("Log day list");
	gtk_misc_set_alignment(GTK_MISC(widget), 0, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, TRUE, 1);
	
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox1), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	widget = gtk_list_new();
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	log_day_list = widget;

	gtk_signal_connect(GTK_OBJECT(log_day_list),
			"select_child",
			nc81_log_day_list_select_child, NULL);
	gtk_signal_connect(GTK_OBJECT(log_day_list),
			"unselect_child",
			nc81_log_day_list_unselect_child, NULL);

	/* construct the log entries list */
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox2), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);

	titles[0] = "Date";
	titles[1] = "Type";
	titles[2] = "Data";
	widget = gtk_clist_new_with_titles(3,titles);
	gtk_clist_set_column_width(GTK_CLIST(widget), 0, 160);
	gtk_clist_set_column_width(GTK_CLIST(widget), 1, 80);
	gtk_clist_set_column_width(GTK_CLIST(widget), 2, 200);
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	log_entries_list = widget;


	/* create the edit menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Delete");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_delete_clicked), NULL);
	button_delete = menu_item;

	nc81_main_set_edit_menu(GTK_MENU(menu));

	/* create the select menu */
	menu = gtk_menu_new();

	menu_item = gtk_menu_item_new_with_label("Tasks");
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_task_list_clicked), NULL);
	gtk_menu_append(GTK_MENU(menu), menu_item);
	button_task_list = menu_item;

	menu_item = gtk_menu_item_new_with_label("Exploitation");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			GTK_SIGNAL_FUNC(button_exploitation_clicked), NULL);
	button_exploitation = menu_item;


	nc81_main_set_select_menu(GTK_MENU(menu));

	nc81_day_view_update();

	return container;
}
/****************************************************************************/
