/* file main.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <gtk/gtk.h>
#include <assert.h>
#include <string.h>
#include "global.h"
#include "icon.h"

static GtkWidget *main_window;
static GtkWidget *actual_form = NULL;
static GtkWidget *form_container;
static GtkWidget *menu_item_edit;
static GtkWidget *menu_item_select;

/****************************************************************************/
void nc81_main_load(nc81_form_type form_type)
{
	GtkWidget *widget = NULL;
	/* remove previous edit menu */
	nc81_main_set_edit_menu(NULL);
	nc81_main_set_select_menu(NULL);
	switch(form_type)
	{
		case nc81_form_main_menu:
			widget = nc81_main_menu_new();
			break;

		case nc81_form_referential_view:
			widget = nc81_ref_view_new();
			break;
		case nc81_form_referential_edit:
			widget = nc81_ref_edit_new();
			break;
		case nc81_form_referential_delete:
			widget = nc81_ref_dele_new();
			break;
		case nc81_form_referential_copy:
			widget = nc81_ref_copy_new();
			break;
		case nc81_form_referential_plan:
			widget = nc81_ref_plan_new();
			break;

		case nc81_form_referential_task_view:
			widget = nc81_ref_task_view_new();
			break;
		case nc81_form_referential_task_edit:
			widget = nc81_ref_task_edit_new();
			break;
		case nc81_form_referential_task_delete:
			widget = nc81_ref_task_dele_new();
			break;
		case nc81_form_referential_task_copy:
			widget = nc81_ref_task_copy_new();
			break;

		case nc81_form_referential_task_dep_view:
			widget = nc81_ref_task_dep_view_new();
			break;
		case nc81_form_referential_task_dep_edit:
			widget = nc81_ref_task_dep_edit_new();
			break;

		case nc81_form_referential_task_operation:
			widget = nc81_ref_task_oper_new();
			break;
		case nc81_form_referential_task_operation_edit:
			widget = nc81_ref_task_oper_edit_new();
			break;
		case nc81_form_referential_task_operation_simu:
			widget = nc81_ref_task_oper_simu_new();
			break;

		case nc81_form_referential_task_condition_view:
			widget = nc81_ref_task_cond_view_new();
			break;
		case nc81_form_referential_task_condition_edit:
			widget = nc81_ref_task_cond_edit_new();
			break;

		case nc81_form_referential_rule_view:
			widget = nc81_ref_rule_view_new();
			break;
		case nc81_form_referential_rule_edit:
			widget = nc81_ref_rule_edit_new();
			break;
		case nc81_form_referential_rule_delete:
			widget = nc81_ref_rule_dele_new();
			break;
		case nc81_form_referential_rule_operation:
			widget = nc81_ref_rule_oper_new();
			break;
		case nc81_form_referential_rule_operation_edit:
			widget = nc81_ref_rule_oper_edit_new();
			break;
		case nc81_form_referential_rule_operation_simu:
			widget = nc81_ref_rule_oper_simu_new();
			break;

		case nc81_form_referential_job_view:
			widget = nc81_ref_job_view_new();
			break;
		case nc81_form_referential_job_edit:
			widget = nc81_ref_job_edit_new();
			break;
		case nc81_form_referential_job_delete:
			widget = nc81_ref_job_dele_new();
			break;

		case nc81_form_referential_job_dep_view:
			widget = nc81_ref_job_dep_view_new();
			break;
		case nc81_form_referential_job_dep_edit:
			widget = nc81_ref_job_dep_edit_new();
			break;

		case nc81_form_exploitation_view:
			widget = nc81_day_view_new();
			break;
		case nc81_form_exploitation_day_edit:
			widget = nc81_day_edit_new();
			break;
		case nc81_form_exploitation_day_delete:
			widget = nc81_day_dele_new();
			break;

		case nc81_form_exploitation_task_condition_view:
			widget = nc81_exp_task_cond_view_new();
			break;

		case nc81_form_exploitation_task_view:
			widget = nc81_exp_task_view_new();
			break;
		case nc81_form_exploitation_task_edit:
			widget = nc81_exp_task_edit_new();
			break;
		case nc81_form_exploitation_task_dep_view:
			widget = nc81_exp_task_dep_view_new();
			break;

		case nc81_form_exploitation_job_view:
			widget = nc81_exp_job_view_new();
			break;
		case nc81_form_exploitation_job_edit:
			widget = nc81_exp_job_edit_new();
			break;
		case nc81_form_exploitation_job_dep_view:
			widget = nc81_exp_job_dep_view_new();
			break;

		case nc81_form_log_day_view:
			widget = nc81_log_day_view_new();
			break;
		case nc81_form_log_day_delete:
			widget = nc81_log_day_dele_new();
			break;
		case nc81_form_log_task_view:
			widget = nc81_log_task_view_new();
			break;
		case nc81_form_log_job_view:
			widget = nc81_log_job_view_new();
			break;

		case nc81_form_sta_ref_view:
			widget = nc81_sta_ref_view_new();
			break;
		case nc81_form_sta_task_view:
			widget = nc81_sta_task_view_new();
			break;
		case nc81_form_sta_job_view:
			widget = nc81_sta_job_view_new();
			break;

		case nc81_form_variable_view:
			widget = nc81_var_view_new();
			break;
		case nc81_form_variable_edit:
			widget = nc81_var_edit_new();
			break;
		case nc81_form_variable_delete:
			widget = nc81_var_dele_new();
			break;


		case nc81_form_user_view:
			widget = nc81_user_view_new();
			break;
		case nc81_form_user_edit:
			widget = nc81_user_edit_new();
			break;
		case nc81_form_user_delete:
			widget = nc81_user_dele_new();
			break;

		case nc81_form_target_view:
			widget = nc81_target_view_new();
			break;
		case nc81_form_target_edit:
			widget = nc81_target_edit_new();
			break;
		case nc81_form_target_delete:
			widget = nc81_target_dele_new();
			break;

		case nc81_form_queue_view:
			widget = nc81_queue_view_new();
			break;
		case nc81_form_queue_edit:
			widget = nc81_queue_edit_new();
			break;
		case nc81_form_queue_delete:
			widget = nc81_queue_dele_new();
			break;

		case nc81_form_monitor:
			widget = nc81_monitor_new();
			break;
		case nc81_form_connection:
			widget = nc81_connection_new();
			break;
		case nc81_form_graph_ref_task:
			widget = nc81_graph_ref_task_new();
			break;
		case nc81_form_graph_ref_job:
			widget = nc81_graph_ref_job_new();
			break;
		case nc81_form_graph_exp_task:
			widget = nc81_graph_exp_task_new();
			break;
		case nc81_form_graph_exp_job:
			widget = nc81_graph_exp_job_new();
			break;

	}
	if (actual_form != NULL)
		gtk_container_remove(GTK_CONTAINER(form_container),actual_form);
	assert(widget != NULL);
	gtk_container_add(GTK_CONTAINER(form_container), widget);
	gtk_widget_show_all(widget);
	actual_form = widget;
}
/****************************************************************************/
void nc81_main_set_edit_menu(GtkMenu *menu)
{
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu_item_edit));
	if (menu != NULL)
	{
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item_edit), 
			GTK_WIDGET(menu));
		gtk_widget_show_all(GTK_WIDGET(menu));
	}
	gtk_widget_set_sensitive(menu_item_edit, (menu==NULL) ? FALSE : TRUE);
}
/****************************************************************************/
void nc81_main_set_select_menu(GtkMenu *menu)
{
	gtk_menu_item_remove_submenu(GTK_MENU_ITEM(menu_item_select));
	if (menu != NULL)
	{
		gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item_select), 
			GTK_WIDGET(menu));
		gtk_widget_show_all(GTK_WIDGET(menu));
	}
	gtk_widget_set_sensitive(menu_item_select, (menu==NULL) ? FALSE : TRUE);
}


/****************************************************************************/
static gboolean goto_callback(GtkMenuItem *menu_item, gpointer data)
{
	nc81_form_type form_type;
	form_type = (nc81_form_type)data;
	nc81_main_load(form_type);
	return TRUE;
}
/*****************************************************************************/
static gboolean menu_quit_clicked(GtkMenuItem* menu_item, gpointer data)
{
	gtk_main_quit();
	return TRUE;
}
/*****************************************************************************/
static gboolean menu_about_clicked(GtkMenuItem* menu_item, gpointer data)
{
	nc81_about_show(main_window);
	return TRUE;
}
/****************************************************************************/
static GtkWidget *nc81_main_make_menu()
{
	GtkWidget *menu_bar;
	GtkWidget *menu_item; 
	GtkWidget *menu;
	menu_bar = gtk_menu_bar_new();

	/* menu file */
	menu_item = gtk_menu_item_new_with_label("File");
	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
	menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu);

	menu_item = gtk_menu_item_new_with_label("Main menu");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_main_menu);

	menu_item = gtk_menu_item_new_with_label("Connection");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_connection);

	menu_item = gtk_menu_item_new_with_label("Referential");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_referential_view);

	menu_item = gtk_menu_item_new_with_label("Exploitation");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_exploitation_view);

	menu_item = gtk_menu_item_new_with_label("Log");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_log_day_view);

	menu_item = gtk_menu_item_new_with_label("Statistics");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_sta_ref_view);



	menu_item = gtk_menu_item_new_with_label("Monitor");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_monitor);

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Variables");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_variable_view);

	menu_item = gtk_menu_item_new_with_label("Targets");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_target_view);

	menu_item = gtk_menu_item_new_with_label("Users");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(goto_callback),
			(gpointer)nc81_form_user_view);

	menu_item = gtk_menu_item_new();
	gtk_menu_append(GTK_MENU(menu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Quit");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(menu_quit_clicked), NULL);

	/* menu edit */
	menu_item = gtk_menu_item_new_with_label("Edit");
	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
	menu_item_edit = menu_item;

	/* menu goto */
	menu_item = gtk_menu_item_new_with_label("Select");
	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
	menu_item_select = menu_item;

	/* help menu */
	menu_item = gtk_menu_item_new_with_label("Help");
	gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), menu_item);
	menu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), menu);

	menu_item = gtk_menu_item_new_with_label("About...");
	gtk_menu_append(GTK_MENU(menu), menu_item);
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate", 
			GTK_SIGNAL_FUNC(menu_about_clicked), NULL);


	return menu_bar;
}
/****************************************************************************/
static void event_destroy(GtkWidget *widget, GdkEvent *event, gpointer data)
{
}
/****************************************************************************/
static void event_delete(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gtk_main_quit();
}
/****************************************************************************/
int main(int argc, char **argv)
{
	int i;
	GtkWidget *vbox;
	GtkWidget *widget;
	/*
	GtkWidget *scrolled;
	*/
	gtk_init(&argc, &argv);

	/* option -s : small interface */
	small = FALSE;
	for (i = 0; i < argc; i++)
		if (strcmp(argv[i], "-s") == 0)
			small = TRUE;
	

	nc81_set_user("admin");
	nc81_set_password("");
	nc81_set_server("localhost");
	nc81_set_ip_port("9993");

	main_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	vbox = gtk_vbox_new(FALSE, 0);

	if (gdk_screen_width() < 300 || gdk_screen_height() < 300)
		small = TRUE;

	if (small)
		gtk_widget_set_usize(main_window, 240, 260);
	else
		gtk_widget_set_usize(main_window, 400, 300);

	gtk_container_add(GTK_CONTAINER(main_window), vbox);

	widget = nc81_main_make_menu(); 
	gtk_box_pack_start(GTK_BOX(vbox), widget, FALSE, FALSE, 0);
	form_container = vbox;
	
	gtk_window_set_title(GTK_WINDOW(main_window), "Narcisse 81");
	gtk_widget_show(main_window);
	nc81_icon_load(main_window);

	nc81_main_load(nc81_form_main_menu);

	gtk_widget_show_all(main_window);

	gtk_signal_connect(GTK_OBJECT(main_window), "delete_event",
					event_delete, NULL);
	gtk_signal_connect(GTK_OBJECT(main_window), "destroy",
					event_destroy, NULL);
	gtk_main();
	return 0;

}
/****************************************************************************/
