/* file mainmenu.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <gtk/gtk.h>
#include <assert.h>
#include "global.h"
#include "icon.h"

/*****************************************************************************/
static gboolean button_referential_clicked(GtkWidget* button, gpointer data)
{
	nc81_main_load(nc81_form_referential_view);
	return TRUE;
}
/*****************************************************************************/
static gboolean button_exploitation_clicked(GtkWidget* button, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_view);
	return TRUE;
}
/*****************************************************************************/
static gboolean button_monitor_clicked(GtkWidget* button, gpointer data)
{
	nc81_main_load(nc81_form_monitor);
	return TRUE;
}
/*****************************************************************************/
static gboolean button_sta_clicked(GtkWidget* button, gpointer data)
{
	nc81_main_load(nc81_form_sta_ref_view);
	return TRUE;
}
/*****************************************************************************/
static gboolean button_log_clicked(GtkWidget* button, gpointer data)
{
	nc81_main_load(nc81_form_log_day_view);
	return TRUE;
}
/*****************************************************************************/
static gboolean button_quit_clicked(GtkWidget* button, gpointer data)
{
	gtk_main_quit();
	return TRUE;
}

/*****************************************************************************/
static GtkWidget *make_presentation_box()
{
	GtkWidget* hbox;
	GtkWidget* widget;

	hbox = gtk_hbox_new(FALSE,0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 10);

	/* icon */
	widget = gtk_pixmap_new(
			nc81_icon_pixmap_array[nc81_icon_diogene87], 
			nc81_icon_bitmap_array[nc81_icon_diogene87]);
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE,0);

	/* presentation */
	widget = gtk_label_new("Narcisse81 is a GUI\n"\
			"for Diogene87,\na powerful job \nscheduler system");
	gtk_label_set_line_wrap(GTK_LABEL(widget), TRUE);
	gtk_label_set_justify(GTK_LABEL(widget), GTK_JUSTIFY_FILL);
	gtk_box_pack_end(GTK_BOX(hbox), widget, TRUE, TRUE,1);

	return hbox;
}

/*****************************************************************************/
static GtkWidget *make_button_box()
{
	GtkWidget* table;
	GtkWidget *button;

	table = gtk_table_new(1, 2, TRUE);

	button = nc81_button_new_with_icon("Referential",
						nc81_icon_referential);
	gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 0, 1);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_referential_clicked), NULL);

	button = nc81_button_new_with_icon("Exploitation", nc81_icon_day);
	gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 1, 2);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_exploitation_clicked), NULL);

	button = nc81_button_new_with_icon("Monitor", nc81_icon_monitor);
	gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 2, 3);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_monitor_clicked), NULL);

	button = gtk_button_new_with_label("Statistics");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 0, 1);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_sta_clicked), NULL);

	button = gtk_button_new_with_label("Log");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1, 2);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_log_clicked), NULL);

	button = gtk_button_new_with_label("Quit");
	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2, 3);
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(button_quit_clicked), NULL);

	return table;
}

/*****************************************************************************/
GtkWidget* nc81_main_menu_new(void)
{
	GtkWidget *vbox;
	GtkWidget *widget;

	vbox = gtk_vbox_new(FALSE,1);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);

	/* presentation */
	widget = make_presentation_box();
	gtk_box_pack_start(GTK_BOX(vbox), widget, TRUE, TRUE,1);

	/* create buttons */
	widget = make_button_box();
	gtk_box_pack_end(GTK_BOX(vbox), widget, FALSE, FALSE,1);

	gtk_widget_show_all(vbox);
	return vbox;
}
/*****************************************************************************/
