/* file ref_plan.c */
/*
 *      narcisse81 - a graphical frontend for diogene87 
 *	Copyright (C) 2005 - Dominique Parisot
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA
 *
 */


#include <assert.h>
#include <string.h>
#include <gtk/gtk.h>
#include <time.h>
#include <stdlib.h>
#include "global.h"
#include "command.h"

static GtkWidget *notebook;
static GtkWidget *field_year;
static GtkWidget *field_month;
static GtkWidget *field_day;
static GtkWidget *calendar;
static GtkWidget *report;

/****************************************************************************/
static void report_callback(gchar** fields, void* data)
{
	gchar* str;
	GList *item_list;
	GtkWidget *item;
	str = g_strjoinv(" ", fields);
	/* create a list item */
	item = gtk_list_item_new_with_label(str);
	gtk_widget_show(GTK_WIDGET(item));
	/* add this item in the list */
	item_list = g_list_append(NULL, item); 
	gtk_list_append_items(GTK_LIST(report), item_list);
	g_free(str);
}
/****************************************************************************/
static gint button_view_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_exploitation_view);
	return 0;
}
/****************************************************************************/
static gint button_ok_clicked(GtkWidget *widget, gpointer data)
{
	gchar* day_name;
	gchar* str_date;
	gint year, month, day;
	gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 1);
	year = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(field_year));
	g_print("year:%i\n", year);
	month = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(field_month));
	day = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(field_day));
	/* clear the previous report */
	gtk_list_clear_items(GTK_LIST(report), 0, -1);
	/* plan the referential */
	str_date = g_strdup_printf("%04i-%02i-%02i", year, month, day);
	nc81_command_send("rp \"%r\" \"%x\"", 
		report_callback,
		NULL, str_date, NULL, NULL);
	g_free(str_date);

	/* select the exploitation day */
	day_name = g_strdup_printf("%s_%04i%02i%02i", 
			nc81_ref_name, year, month, day);
	nc81_day_select(day_name);
	g_free(day_name);

	/*
	nc81_main_load(nc81_form_exploitation_view);
	*/
	return 0;
}

/****************************************************************************/
static gint button_cancel_clicked(GtkWidget *widget, gpointer data)
{
	nc81_main_load(nc81_form_referential_view);
	return 0;
}
/****************************************************************************/
gboolean calendar_day_selected(GtkWidget *widget, gpointer data)
{
	gint year, month, day;
	gtk_calendar_get_date(GTK_CALENDAR(widget), &year, &month, &day);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(field_year), year);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(field_month), month + 1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(field_day), day);
	return TRUE;
}

/****************************************************************************/
/* this function creates a widget to plan a rerential*/
GtkWidget* nc81_ref_plan_new(void)
{
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *bbox;
	GtkWidget *widget;
	GtkWidget *scrolled;
	GtkObject *adj;
	gint year, month, day;
	time_t start_time;
	struct tm dc;

	assert(nc81_ref_name != NULL);

	notebook = gtk_notebook_new();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox,
			gtk_label_new("Parameters"));

	/* the referential name*/
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Referential");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	gtk_misc_set_alignment(GTK_MISC(widget), 0.5, 0);
	widget = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	gtk_entry_set_editable(GTK_ENTRY(widget), FALSE);
	gtk_entry_set_text(GTK_ENTRY(widget), nc81_ref_name);

	/* the start time */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
	widget = gtk_label_new("Start time");
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	gtk_misc_set_alignment(GTK_MISC(widget), 0.5, 0);

	/* year */
	adj = gtk_adjustment_new(2000, 2000, 2100 , 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 60, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	gtk_widget_grab_focus(widget);
	field_year = widget;

	/* month */
	adj = gtk_adjustment_new(1, 1, 12, 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 40, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	gtk_widget_grab_focus(widget);
	field_month = widget;

	/* day */
	adj = gtk_adjustment_new(1, 1, 31 , 1, 2, 0);
	widget = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.2, 0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(widget), TRUE);
	gtk_widget_set_usize(widget, 40, 0);
	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 1);
	gtk_widget_grab_focus(widget);
	field_day = widget;

	/* the calendar */
	hbox = gtk_hbox_new(FALSE, 2);
	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 1);
	widget = gtk_calendar_new();
	gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 1);
	calendar = widget;
	gtk_signal_connect(GTK_OBJECT(calendar), "day_selected",
			GTK_SIGNAL_FUNC(calendar_day_selected), NULL);

	/* button box */
	bbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_start(GTK_BOX(vbox), bbox, TRUE, TRUE, 1);

	/* create button */
	widget = gtk_button_new_with_label("Create");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_ok_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(bbox), widget);

	/* close button */
	widget = gtk_button_new_with_label("Close");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(bbox), widget);

	/* report page */
	vbox = gtk_vbox_new(FALSE, 2);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox,
			gtk_label_new("Report"));
	/* create the report */
	scrolled = gtk_scrolled_window_new(NULL,NULL);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 1);
	gtk_scrolled_window_set_policy(
			GTK_SCROLLED_WINDOW(scrolled),
			GTK_POLICY_AUTOMATIC,
			GTK_POLICY_AUTOMATIC);
	widget = gtk_list_new();
	gtk_scrolled_window_add_with_viewport(
			GTK_SCROLLED_WINDOW(scrolled),
			widget);
	report = widget;

	/* button box */
	bbox = gtk_hbutton_box_new();
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
	gtk_box_pack_start(GTK_BOX(vbox), bbox, TRUE, TRUE, 1);

	/* view button */
	widget = gtk_button_new_with_label("View day");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_view_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(bbox), widget);

	/* close button */
	widget = gtk_button_new_with_label("Close");
	gtk_signal_connect(GTK_OBJECT(widget), "clicked",
			GTK_SIGNAL_FUNC(button_cancel_clicked), NULL);
	gtk_container_add(GTK_CONTAINER(bbox), widget);

	/* select next day */
	start_time = time (0) + 24 * 3600;
	dc = *localtime (&start_time);
	dc.tm_hour = 0;
	dc.tm_min = 0;
	dc.tm_sec = 0;
	start_time = mktime (&dc);
	year = dc.tm_year + 1900;
	month = dc.tm_mon + 1;
	day = dc.tm_mday;

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(field_year), year);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(field_month), month);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(field_day), day);

	gtk_calendar_select_month(GTK_CALENDAR(calendar), month - 1, year);
	gtk_calendar_select_day(GTK_CALENDAR(calendar), day);

	return notebook;
}
/****************************************************************************/
